/** \file ActorDomainRecord.cpp 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "ActorDomainRecord.h"

namespace rtaixml_domain
{

	/**
	 * Default constructor
	 * @return 
	 */
	ActorDomainRecord::ActorDomainRecord()
	{
		sSlaveName   = "";
		sSessionId   = "";
		sTargetName  = "";
		iPortXML     = 0;
		sIP 			 = "127.0.0.1";
	}
	
	/**
	 * Default destructor
	 * @return 
	 */
	ActorDomainRecord::~ActorDomainRecord()
	{
	}
	
	/**
	 * Standard Constructor
	 * @param sSlaveName_new name of the slave server
	 * @param sSessionId_new session identifier
	 * @param sTargetName_new target name
	 * @param iPortXML_new XMLRPC port
	 * @param sIP_new IP where rtaixml and the target are running (default 127.0.0.1)
	 * @return 
	 */
	ActorDomainRecord::ActorDomainRecord(std::string sSlaveName_new,std::string sSessionId_new,
										std::string sTargetName_new,int iPortXML_new,std::string sIP_new="127.0.0.1")
	{
		sSlaveName   = sSlaveName_new;
		sSessionId   = sSessionId_new;
		sTargetName  = sTargetName_new;
		if(!setIP( sIP_new ))
			setIP("127.0.0.1");
	
		if(iPortXML_new>1024 && iPortXML_new<65537)
			iPortXML = iPortXML_new;
		else
			iPortXML = DEFAULT_BASE_PORT;;
			
	}
	
	/**
	 * Get the slave name
	 * @return slave name
	 */
	std::string ActorDomainRecord::getSlaveName()
	{
		return sSlaveName;
	}
	
	/**
	 * Set the slave name
	 * @param sSlaveName_new New slave name 
	 */
	void ActorDomainRecord::setSlaveName(std::string sSlaveName_new)
	{
		sSlaveName   = sSlaveName_new;
	}
			
	/**
	 * Get the session identifier
	 * @return the session identifier related to this actor
	 */
	std::string ActorDomainRecord::getSessionId()
	{
		return sSessionId;
	}
	
	/**
	 * Set the session id
	 * @param sSessionId_new new session identifier
	 */
	void ActorDomainRecord::setSessionId(std::string sSessionId_new)
	{
		sSessionId   = sSessionId_new;
	}
	
	/**
	 *   Get the target name
	 * @return the target name
	 */
	std::string ActorDomainRecord::getTargetName()
	{
		return sTargetName;
	}
	
	/**
	 * Set the target name
	 * @param sTargetName_new New target name
	 */
	void ActorDomainRecord::setTargetName(std::string sTargetName_new)
	{
		sTargetName  = sTargetName_new;
	}
	
	/**
	 * Get the XMLRPC port
	 * @return XMLRPC port
	 */
	int ActorDomainRecord::getPortXML()
	{
		return iPortXML;
	}
	
	/**
	 * Set a new XMLRPC port
	 * @param iPortXML_new new XMLRPC port
	 */
	void ActorDomainRecord::setPortXML(int iPortXML_new)
	{
		if(iPortXML_new>1024 && iPortXML_new<65537)
			iPortXML = iPortXML_new;
		else
			iPortXML = DEFAULT_BASE_PORT;;
	}

	/**
	 * Get the IP
	 * @return IP
	 */
	std::string ActorDomainRecord::getIP()
	{
		return sIP;
	}

	/**
	 * Set a new IP
	 * @param sNewIP new IP
	 * @return 
	 */
	bool ActorDomainRecord::setIP(std::string sNewIP)
	{
		StringTokenizer strtok = StringTokenizer(sNewIP,".");
		int iCnt = strtok.countTokens();
		if(iCnt!=4)
			return false;
		for(int i = 0; i < iCnt; i++)
		{
			std::string sTemp =strtok.nextToken();
			try{atoi(sTemp.c_str());}
			catch(...)
			{return false;}
		}
		return true;
	}
}
