/** \file RTDomain.h 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef RTDOMAIN_H_
#define RTDOMAIN_H_


/* Class Name:		RTDomain
 * Author: 			Marco Romagnoli
 * Description:	Info about the real time domain.
 */

#include "ActorDomainRecord.h"
#include "CheckPoint.h"
#include "CheckPointFake.h"
#include "StringTokenizer.h"
#include "Constants.h"
#include "SlaveInfo.h"

#include <cstdlib>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sys/wait.h>
#include <sys/types.h>

#include <stdio.h>
#include <pthread.h>
#include <sys/poll.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/io.h>
#include <sys/poll.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <vector>
#include <string>

namespace rtaixml_domain
{
  
class RTDomain
{
	// Constructor and Destructor protected per Singleton
	public:
		RTDomain& operator= (const RTDomain&);
		virtual ~RTDomain();
		
		//if true - debug mode on
		const static bool IS_DEBUG = false;
				
	protected:

		std::string sMasterName;
		CheckPoint *cpSecure;
		//This is the XML port of the master, slave port are the next
		int iBasePortXML;
		int iMaxPortXML;
		
		//This is the path to the file containing all the real time names.
		std::string sRtaiNames;
		
		//Used to get the session id and the slave server name
		std::string get_random_string(unsigned int);
		//Used to parse /prov/rtai/names
		bool parse_line(std::string,std::string);
		//Provides a free socket port inside the reserved interval.
		virtual int GetFreePort() = 0;
		
		//Initialize the parameter from rtaixml.config. This method MUST be called from any child of RTDomain
		void Initialize();
		
	//Services
	public:
		virtual unsigned int NumberOfActiveProcesses() = 0;
		//An actor is a pair (slave_server,target) with all the related information
		virtual ActorDomainRecord GetActor(std::string) = 0;
		//short: 0 - SlaveName, 1 - SessionId, 2 - TargetName, 3 - ScopeName, 4 - DigitalName, 5 - PortXML <-- OLD 
		//short: 0 - SlaveName, 1 - SessionId, 2 - TargetName, 3 - ScopeName, 4 - DigitalName, 5 - PortXML <-- OK!
		virtual ActorDomainRecord GetActor(std::string,short) = 0;
		virtual std::string GetFreeSlaveName() = 0;
		virtual std::string GetFreeSessionId() = 0;
		
		virtual bool AddActor(ActorDomainRecord) = 0;
		virtual bool AddActorAndChoosePort(ActorDomainRecord) = 0;
		//the parameters are (in order):  sTargetName sScopeName sDigitalName <-- OLD
		//the parameters are (in order):  sTargetName sScopeName sDigitalName <-- OK!
		virtual bool AddNewActorAndChoosePort(std::string/*,std::string,std::string*/) = 0;
		
		virtual bool RemoveActor(ActorDomainRecord) = 0;
		//The parameter is the TaskName
		virtual bool RemoveActor(std::string) = 0;
		virtual bool RemoveAllActor(ActorDomainRecord) = 0;
		
		virtual bool StopRemoveSlaveServer(std::string) =0;
		
		//Manage the XML port of the master server
		int getPortXML();
		int getMaxPortXML();
		bool setBasePortXML(int);
		bool setMaxPortXML(int);
		//The parameter is the TaskName
		virtual std::string getSessionId(std::string) = 0;
		//The parameter is the Session Id
		virtual std::string getTaskName(std::string) = 0;
		//The second paramter is 0 if the first id the TaskName, 1 if the first parameter is the SessionId
		virtual std::string getSlaveName(std::string,short) = 0;
		//The second paramter is 0 if the first id the TaskName, 1 if the first parameter is the SessionId
		virtual int getPortXMLSlave(std::string,short) = 0;
		//Get a vector containing all the slave names and the xmlrpc port
		virtual std::vector< SlaveInfo > getAllSlaveInfo() = 0;
		
		
		//Manage the server master name
		std::string getMasterName();
		void setMasterName(std::string);
		
		//Checks in /proc/rtai/names if there is a target with the provided name. I can not use RT_get_address of rtai because I want to 
		// keep the master server a non real time applications.
		bool IsThereATarget(std::string);
		
		//Security check (TBD). It exploits the architectural pattern Check Point that can be used in combination with Roles, Single
		// Access Point (already implemented), Session, Full/Limited view to provide an High Application Security level.
		bool SecureCheck();
		
		//Print all the available information about the real time domain in a human readable format on the standard output.
		virtual void PrintCurrentStatus() = 0;

		//Check if there is already a slave server that manages the selected target. parameter: targetname.
		virtual bool CheckTargetSlaveManager(std::string) = 0;
		
		//Check if there is some slaves without an enabled target waiting for client interaction. The target name will be put to blank.
		virtual int ClearDomain() = 0;
		
		//Check if the target (taskname,scopename,digitalname) is already present in the real time domain. <-- OK!
		virtual bool IsTargetPresent(std::string) = 0;

		void LoadConfigFile();
};

}

#endif /*RTDOMAIN_H_*/
