/** \file rtaixml.cpp
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)
*
*This library is free software; you can redistribute it and/or
*modify it under the terms of the GNU Lesser General Public
*License as published by the Free Software Foundation; either
*version 2 of the License, or (at your option) any later version.
*
*This library is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*Lesser General Public License for more details.
*
*You should have received a copy of the GNU Lesser General Public
*License along with this library; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
**/


#include "ServerSocket.h"
#include "SocketException.h"
#include "XmlRpc.h"
#include "Signal_Manager.h"
#include "Digital_Manager.h"
#include "RTDomain.h"
#include "slave.h"

#include <string>
#include <sstream>

#include <semaphore.h> 

#include <iostream>
#include <stdio.h>
#include <pthread.h>
#include <sys/poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <sys/io.h>
#include <sys/poll.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>


// #include <rtai_netrpc.h>
#include <rtai_msg.h>
#include <rtai_mbx.h>


using namespace XmlRpc;
//using namespace rtaixml_domain;
namespace rtaixml_domain {

	

void rlg_quit_cb(void);
void rlg_connect_cb(void);
void rlg_disconnect_cb(void);
void rlg_read_pref(void);
void rlg_start_stop_cb(void);
void rlg_update_parameters_cb(int,int, double );
void rlg_upload_parameters_cb(void);
double get_parameter(Target_Parameters_T, int, int, int *);
int SignalNumber(void);

//XmlRpc Server variable
XmlRpcServer s;
// XML-RPC default server port
int portXML = DEFAULT_BASE_PORT;
// bSlave=true if the server runs in a master-server mode otherwise in standalone mode
bool bSlave = false;
// bConnectionAvailable=true if the server accept new connection.
bool bConnectionAvailable = true;
//Let the complete shutdown of the server (default in slave mode) after a DISCONNET or a STOP
bool bCompleteShutdown = false;
//Target Interface class (rtmain version): 0 - pre RTAI 3.4 / 1 - RTAI 3.4
int iTargetInterfaceClass = 1;

//Server ID provided by the master server or set by input parameters.
std::string sIdServer = "ServerId";
//Intentionally not initialized. It has to be initialized in the main after the parameters processing.
std::string sIdSession;

//XML port of the master server
int MasterXMLPort=0;
//Name of the master server
std::string MasterServerName="";

//Debug flag
bool IS_DEBUG = true;
//Set if the STOP_TARGET comes from an xmlrpc request (false) or from an automatic stop target (true).
bool StopRequestFromTarget = true;

int portSocket=DEFAULT_BASE_PORT+1;	//Default Socket port transmission
bool client_connected=false;
bool client_waiting=false;

//mutex to manage socket data sent to the client
sem_t mutex;
sem_t mutex_sock;
sem_t mutex_close;
ServerSocket listen_sock;
ServerSocket sending_sock;

static RT_TASK *Target_Interface_Task;
static RT_TASK *RLG_Main_Task;
static pthread_t Target_Interface_Thread;
static pthread_t *Get_Scope_Data_Thread;
static pthread_t *Get_Led_Data_Thread;
static pthread_t Send_Data_Target_Thread;

static int Verbose = 0;

static int Is_Target_Connected;
static unsigned int Is_Target_Running;
static int End_App;
static int End_All;

int Num_Tunable_Parameters;
int Num_Tunable_Blocks;
int Num_Scopes,Num_Leds;

Target_Parameters_T *Tunable_Parameters;
Target_Blocks_T *Tunable_Blocks;
Target_Scopes_T *Scopes;
Target_Logs_T *Logs;
Target_ALogs_T *ALogs;
Target_Leds_T *Leds;
Target_Meters_T *Meters;
Target_Synchs_T *Synchs;

Batch_Parameters_T Batch_Parameters[MAX_BATCH_PARAMS];
Preferences_T Preferences;
Profile_T *Profile;

Signal_Manager *Signals_Manager;
Digital_Manager *Digitals_Manager;

const char *RLG_Target_Name;

static std::string ServerName("RLGM2");
static std::string TargetName("DEFAULT_TARGET_NAME");
static std::string ScopeMbxName("DEFAULT_SCOPE_NAME");
static std::string DigitalMbxName("DEFAULT_DIGITAL_NAME");
static std::string target_script("rtarget");


static volatile int GlobalRet[16];

/** \brief It sends a message to the specified task
 * 
 * This function let you  to send a message msg (unsigned int) to the task 
 * specified. It is used to send a message to the target interface thread.
 * @param task the name of the task you want to communicate with.
 * @param msg message sent to the task. It's an unsigned int.
 * @param reply unused.
 */
static inline void RT_RPC(RT_TASK *task, unsigned int msg, unsigned int *reply)
{
    GlobalRet[msg & 0xf] = 0;
    rt_send(task, msg);
    while (!GlobalRet[msg & 0xf]) {
        msleep(100);            //to force a behavior similar to Fl::wait(time);
    }
}


static inline void RT_RETURN(RT_TASK *task, unsigned int reply)
{
    GlobalRet[reply] = 1;
}

/** \brief It obtains an unique id from a name.
 * 
 * This function let you to obtain an unique id from a name. It gets the name and 
 * attached to it an integer and then try to register this name inside the
 * RTAI domain.
 * @param root the name used to get the unique id
 * @return the unique RTAI id in case of success, elsewhere 0.
 * 
 */
unsigned long get_an_id(const char *root)
{
    int i;
    char name[7];
    for (i = 0; i < MAX_NHOSTS; i++) {
        sprintf(name, "%s%d", root, i);
        if (!rt_get_adr(nam2num(name))) {
            return nam2num(name);
        }
    }
    return 0;
}



/** \brief It creates a random string of any lenght.
 * 
 * This function let you to obtain a random string. It is used to create session 
 * identifiers
 * @param lenght: lenght of the string.
 * @return the random string generated.
 * 
 */std::string get_random_string(unsigned int lenght)
{
	std::string random_string(lenght,'x');
	int random_integer;
	
	//pause to improve the random generation
	clock_t start_time = clock();
	while((clock() - start_time) < 0.1 * CLOCKS_PER_SEC) {}
	
	for(unsigned int index=0; index<lenght; index++){
		random_integer = (rand()%36);
		//Numbers
		if(random_integer<10)
			random_string[index]=48+random_integer;
		//Capitol Letter
		else if(random_integer<36)
			random_string[index]=65+random_integer-10;
		else
			random_string[index]=65;
		//Small Letter
		//else if(random_integer<62)
		//	random_string[index]=97+random_integer-36;
		//else
		//	random_string[index]=97;
	}
	return random_string;
}
	
/** \brief It checks if a target exists.
 * 
 * 	This function let you know if a target with the specified name exists. It 
 * check if the corresponding name is registered in the RTAI domain.
 * @param target_name the name of the target or any RTAI name you want to check
 * @return true if the name is registered elsewhere false.
 */

bool Is_There_A_Target(const std::string target_name)
{
// 	std::cout << "PIPPO: " << rt_get_adr(nam2num(target_name.c_str())) << " PLUTO!";
	
	if (rt_get_adr( nam2num(target_name.c_str()) ) )
		return true;
	return false;
}

/** @defgroup slave Slave Server (rslave)
 *  This group contains all the Remote Procedure Call available via XMLRPC to deal with the Slave Server.
 *  If the Slave Server runs in master/slave mode (rtaixml started with option -M) Connection_Request is not
 *  available.
 *  @{
 */

/** \class Connection_Request_.
 * \brief Request a connection to the rtaixml domain (standalone).
 * 
 * This method is available as web service. It allows you to request a connection to 
 * the rtaixml server in stand alone mode. Otherwise this function is disable. 
 * establish a connection
 * between the specified target and the server. This operation is necessary before 
 * doing any other action on the target.
 * <div class="fragment"><div class="fragment1">
 * <span class="preprocessor"><b>IMPORTANT! READ CAREFULLY!</b></span><br>
 * <span class="">The XMLRPC method Connection_Request is located in rtaixml.cpp and in 
 * rtaixml_master.cpp because the framework can operate in master/slave mode and in standalone mode.
 * The methods have the same interface but work differently.</span><br>
 * <span class="keyword">For more info see page \ref master_slave. </span>
 * <span class="keyword">See also the class Connection_Request. </span><br>
 * </div></div>
 * <div class="fragment"><div class="fragment1">
 * <span class="preprocessor"><b>Doxygen failure!</b></span><br>
 * <span class="">The class name in the documentation is Connection_Request_ but the real class name in 
 * rtaixml.cpp is Connection_Request because Doxygen has some problems in dealing with such names. </span><br>
 * </div></div>
 * 
 * @param params it has to contain as element 0 the session identifier as provided
 * by ConnectionRequest
 * @param result This variable contains the result of the execution of this 
 * function. It contains three values. The first "str" is an human readable
 * string that explains what's happened. The second "state" is an integer value
 * that contains the state of the system (target+server) and can be explained in 
 * this way. The first four bits contains the useful information. The less one 
 * indicates if an error occurred, the second shows if the target exists, the third
 * let you know if the target is connected to the server and the last if the
 * connected target is running. The last "port" contains the port number you
 * have to communicate with to obtain the signals.
 * 
 * @return see parameter result. 
 */
 class Connection_Request : public XmlRpcServerMethod
{
public:
	Connection_Request(XmlRpcServer* s) : XmlRpcServerMethod("Connection_Request", s) {}

	void execute(XmlRpcValue& params, XmlRpcValue& result)
	{
		if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
		if(IS_DEBUG) std::cout << ">> Connection_Request <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		XmlRpcValue status;

		if(IS_DEBUG) std::cout << "## Checking..." << std::endl;
		if(bSlave || !bConnectionAvailable) {
			if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
			status["id_session"]= "";
			status["state"]=(int)1;
			if(bSlave)
				status["str"]="CONNECTION_REQUEST is not allowed in Slave mode.";
			else
				status["str"]="CONNECTION_REQUEST is disables.";
			result=status;
			if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
			if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
			return;
		}
		if(IS_DEBUG) std::cout << "Autonomous mode active." << std::endl; 		

		if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
		//Validity check.
		// Formal check
		if(IS_DEBUG) std::cout << "** Formality...";
		if(!params.valid() || params.size()<1 ) {
			if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
			status["id_session"]= "";
			status["state"]=(int)1;
			status["str"]="Number of parameters in calling CONNECTION_REQUEST incorrect";
			result=status;
			if(IS_DEBUG) std::cout << ">> End of Connection_Request <<" << std::endl;
			if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
			return;
		}
		if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		

		TargetName=std::string(params[0]);

		sIdSession = get_random_string(DEFAULT_SESSION_LENGHT);
		
		if(IS_DEBUG) std::cout << "-----" << std::endl; 		
		if(IS_DEBUG) std::cout << "Target Name: " << TargetName << std::endl; 		
		if(IS_DEBUG) std::cout << "Session ID: " << sIdSession << std::endl; 		
		if(IS_DEBUG) std::cout << "-----" << std::endl; 		

		//return values setting. id_session and port necessary to contact the slave server.
		status["id_session"] = sIdSession;
		status["state"]      = (int)0;
		status["str"]        = "Connection accepted.";
		status["port"]       = portXML;
		result=status;

		if(IS_DEBUG) { std::cout << ">> End of Connection_Request <<" << std::endl; std::fflush(stdout); }
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;

		bCompleteShutdown = false;
	}
} connection_request(&s);


/** \class tConnect
 * \brief Connect the server to the target via XMLRPC.
 * 
 * This method is available as web service. It allows you to establish a connection
 * between the specified target and the server. This operation is necessary before 
 * doing any other action onthe target.
 * @param params it has to contain as element 0 the session identificator as provided
 * by ConnectionRequest
 * @param result This variable contains the result of the execution of this 
 * function. It contains three values. The first "str" is an human readable
 * string that explains what's happened. The second "state" is an integer value
 * that contains the state of the system (target+server) and can be explained in 
 * this way. The first four bits contains the useful information. The less one 
 * indicates if an error occurred, the second shows if the target exists, the third
 * let you know if the target is connected to the server and the last if the
 * connected target is running. The last "port" contains the port number you
 * have to communicate with to obtain the signals.
 * 
 * @return see parameter result. 
 */
class tConnect : public XmlRpcServerMethod
{
public:
tConnect(XmlRpcServer* s) : XmlRpcServerMethod("tConnect", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> tConnect <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	std::string temp;

	//Validity check.
	if(!params.valid() || params.size()!=1 ) {
		status["str"]="Number of parameters in calling CONNECT incorrect";
		status["state"]=(int)1;
		status["port"]=(int)0;
		result=status;
		return;
	}

	//Check Session id
	if(std::string(params[0]).compare(sIdSession)) {
		status["str"]="ID_session incorrect.";
		status["state"]=(int)1;
		status["port"]=(int)0;
		return;
	}
	
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No target or bad interface task identifier associated to "+std::string(params[0])+" session.";
		//Target not exists
		status["state"]=(int)1;
		status["port"]=(int)0;
		result=status;
		bCompleteShutdown=false;
		rlg_quit_cb();
		result=status;
		bConnectionAvailable = true;
		if(IS_DEBUG) { std::cout << ">> End of tConnect <<" << std::endl; std::fflush(stdout); }
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(Is_Target_Connected)
		RT_RPC(Target_Interface_Task, DISCONNECT_FROM_TARGET, 0);

	if(!Is_Target_Connected)
		RT_RPC(Target_Interface_Task, CONNECT_TO_TARGET, 0);
	else {
		status["str"]="Error.";
		status["state"]=(int)1;
		status["port"]=(int)0;
		result=status;
		return;
	}
		
	status["str"] = Tunable_Parameters[0].model_name;
	//state are 4 bits.
	// Is_Target_Running | Is_Target_Connected | Target_Exist | Error
	status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
	status["port"]=(int)portSocket;
	if((int)Is_Target_Running==1){
		if (Verbose)
			printf("\nResetting scopes\n");
		Signals_Manager->ResetAll();
		Digitals_Manager->ResetAll();
	}
	if(!Is_Target_Connected) {
		status["str"]="Error.";
		status["state"] = (int)1; 
		status["port"]=(int)0;
	}
	result = status;
	bConnectionAvailable=false;

	if(IS_DEBUG) { std::cout << ">> End of tConnect <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	}

std::string help() { return std::string("Allows Target connection"); }

} tconnect(&s);    // This constructor registers the method with the server

/** \class Disconnect
* \brief Disconnect the server from the target via XMLRPC.
* 
* This method is available as web service. It allows you to interrupt a connection
* with the specified target.
* @param params it has to contain as element 0 the task name to disconnect
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
* 
* @return see parameter result. 
*/
class Disconnect : public XmlRpcServerMethod
{
public:
Disconnect(XmlRpcServer* s) : XmlRpcServerMethod("Disconnect", s) {}
void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Disconnect <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	//Validity check.
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling DISCONNECT incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	//Chech task name or session id
	if(Verbose)
		std::cout<<"Disconnect: "<<params[0]<<"\t Valore corretto->"<< sIdSession <<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}

	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	if (Is_Target_Connected) {
		if(Is_There_A_Target(TargetName))
			RT_RPC(Target_Interface_Task, DISCONNECT_FROM_TARGET, 0);
		status["str"]="Disconnected from Target.";
		status["state"]=(int)2;
	}
	else {
		status["str"]="Target not Connected";
		status["state"]=(int)0;
	}	

	bCompleteShutdown=false;
	if(IS_DEBUG) { std::cout << "Close the connection with the target..."; std::fflush(stdout); }
	rlg_quit_cb();
	if(IS_DEBUG) { std::cout << "done." << std::endl; std::fflush(stdout); }

	
	result=status;    
	bConnectionAvailable = true;    
	
	if(IS_DEBUG) { std::cout << ">> End of Disconnect <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows Target disconnection"); }

} disconnect(&s);    // This constructor registers the method with the server


/** \class Info
 * \brief Get infos about the server and the target via XMLRPC.
 * 
 * This method is available as web service. It allows you to get information
 * about the server and the target.
 * @param params it has to contain as element 0 the task name to get infos about
 * @param result This variable contains the result of the execution of this 
 * function. It contains two values. The first "str" is an human readable
 * string that explains what's happened. The second "state" is an integer value
 * that contains the state of the system (target+server) and can be explained in 
 * this way. The first four bits contains the useful information. The less one 
 * indicates if an error occurred, the second shows if the target exists, the third
 * let you know if the target is connected to the server and the last if the
 * connected target is running. The last "port" contains the port number you
 * have to communicate with to obtain the signals.
 * 
 * @return see parameter result. 
 */
class Info : public XmlRpcServerMethod
{
public:
Info(XmlRpcServer* s) : XmlRpcServerMethod("Info", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Info <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	std::string temp;
	//Formal validity
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling INFO incorrect";
		status["state"]=(int)1;
		status["port"]=(int)0;
		result=status;
		return;
	}

	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}

	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		status["port"]=(int)portSocket;
		result=status;
		return;
	}
	if(Verbose)
		std::cout<<"Info: "<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	status["str"] = Tunable_Parameters[0].model_name;
	status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
	status["port"]=(int)portSocket;
	result = status;
	if(IS_DEBUG) { std::cout << ">> End of Info <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	}

std::string help() { return std::string("Get info about target."); }

} info(&s);    // This constructor registers the method with the server


/** \class Start
* \brief Start the specified target via XMLRPC.
* 
* This method is available as web service. It allows you to start a connected
* target. If the specified target is not connected to the server, this function
* return an error.
* @param params it has to contain as element 0 the task name to start
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
* 
* @return see parameter result. 
*/
class Start : public XmlRpcServerMethod
{
public:
Start(XmlRpcServer* s) : XmlRpcServerMethod("Start", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Start <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling START incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Start: "<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	if(Is_Target_Connected)
		if (!Is_Target_Running){
			RT_RPC(Target_Interface_Task, START_TARGET, 0);
			status["str"]="Start Realtime Code";
		}
		else 
			status["str"]="Target already running";
	else
		status["str"]="Target not connected";
	status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
	result=status;
	if(IS_DEBUG) { std::cout << ">> End of Start <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows Target start"); }

} start(&s);    // This constructor registers the method with the server


/** \class Stop
* \brief Stop the specified target via XMLRPC.
* 
* This method is available as web service. It allows you to stop a connected 
* and running target. If the specified target is not connected to the 
* server, this function return an error.
* @param params it has to contain as element 0 the task name to start
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
* 
* @return see parameter result. 
*/
class Stop : public XmlRpcServerMethod
{
public:
Stop(XmlRpcServer* s) : XmlRpcServerMethod("Stop", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling STOP incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Stop: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	if(Is_Target_Connected)
		if (Is_Target_Running){
			if(Is_There_A_Target(TargetName)) {
				StopRequestFromTarget=false;
				RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
			}
			status["str"]="Stop Realtime Code";
		}
		else
			status["str"]="Target not running";
	else 
		status["str"]="Target not connected";
	status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 

	bCompleteShutdown=false;
	if(IS_DEBUG) { std::cout << "Close the connection with the target..."; std::fflush(stdout); }
	rlg_quit_cb();
	if(IS_DEBUG) { std::cout << "done." << std::endl; std::fflush(stdout); }

	result=status;
	bConnectionAvailable = true;
	
	if(IS_DEBUG) { std::cout << ">> End of Stop <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Allows Target stop"); }

} stop(&s);    // This constructor registers the method with the server


/** \class Stop_Server
* \brief Stop the RTAI-XML Server via XMLRPC.
* 
* This method is available as web service. It allows you to stop the server.
* 
* @param params it has to contain as element 0 the server name. When 
* executing the RTAI-XML server is possible to provide a name for the server.
* This name is used to register the soft real time thread that provide the 
* external interface in the RTAI domain. In order to stop the server you have
* to know its name, in this way a little of security is provided.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
* 
* @return see parameter result. 
*/
class Stop_Server : public XmlRpcServerMethod
{
public:
Stop_Server(XmlRpcServer* s) : XmlRpcServerMethod("Stop_Server", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop_Server <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	if(IS_DEBUG) std::cout << "## Checking parameters..." << std::endl;
	
	if(IS_DEBUG) std::cout << "** Formality...";
	if(!params.valid() || params.size()<1 || params.size()>2) {
		if(IS_DEBUG) std::cout << "FAILURE." << std::endl; 		
		status["str"]="Number of parameters in calling STOP_SERVER incorrect";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Stop_Server <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. " << params.size() << " parameter(s) received." << std::endl; 		

	// 1. legality check
	if(IS_DEBUG) std::cout << "** Legality...";
	if(std::string(params[0]).compare(sIdServer)) {
		if(IS_DEBUG) std::cout  << "FAILURE -- " << "Server/Session Id provided: "<< params[0] << " - sIdSession correct: " << sIdSession << std::endl; 		
		status["str"]="Server/Session Id incorrect.";
		status["state"]=(int)1;
		result=status;
		if(IS_DEBUG) std::cout << ">> End of Stop_Master_Server <<" << std::endl;
		if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
		return;
	}
	if(IS_DEBUG) std::cout << "OK. MasterName: " << ServerName << std::endl; 		

	//If the stop server comes from a master server it must to disable the slave mode otherwise in rlg_quit_cb you do an xmlrpc request to the
	// master server and it creates stall.
	if(params.size()==2)
		bSlave=false;

	bCompleteShutdown=true;
		
	if(IS_DEBUG) std::cout << "Close the connection with the target..."; 		
	std::fflush(stdout);
	rlg_quit_cb();
	if(IS_DEBUG) std::cout << "done." << std::endl; 		
	std::fflush(stdout);

	status["str"]="Server stopped.";
	status["state"]=(int)0;
	result=status;

	if(IS_DEBUG) std::cout << ">> End of Stop_Server <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	std::fflush(stdout);
}

std::string help() { return std::string("Allows Server stop"); }

} stop_server(&s);    // This constructor registers the method with the server


/** \class Get_Param
* \brief Get the parameters of the connected target via XMLRPC.
* 
* This method is available as web service. It allows you to get the parameters
* of a connected target. If the specified target is not connected to the 
* server, this function return an error.
* @param params it has to contain as element 0 the task name.
* @param result This variable contains the result of the execution of this 
* function. It is a vector of XmlRpcValue element of size number of parameters 
* +1. The first element contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running. The remaining elements contain the parameters. Each 
* element contains a parameters. Each parameter is an XmlRpcValue formed by
* 4+parameter dimension elements.The first is the block name, the second is the
* parameter name, the third is the number of rows the fourth is the number of
* columns, the last (number of rows * number of column) values contain the 
* numerical values of the parameters.
* If an error occurred only a one value vector is returned with the first 
* specified element.
* 
* @return see parameter result. 
*/
class Get_Param : public XmlRpcServerMethod
{
public:
Get_Param(XmlRpcServer* s) : XmlRpcServerMethod("Get_Param", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Get_Param <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling GET_PARAMS incorrect";
		status["state"]=(int)1;
		result[0]=status;
		return;
	}
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result[0]=status;
		return;
	}
	
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result[0]=status;
		return;
	}
	XmlRpcValue Param_XML;
	XmlRpcValue struct1;
	RT_RPC(Target_Interface_Task, GET_PARAMS, 0);
	//+1 is needed for the state
	Param_XML.setSize(Num_Tunable_Parameters+1);
	status["str"]="Parameters ok.";
	status["state"]=(int)((Is_Target_Running*8+Is_Target_Connected*4+2));
	Param_XML[0]=status;
	struct1.setSize(5);
	for (int n = 0; n < Num_Tunable_Parameters; n++) {
		struct1[0] = Tunable_Parameters[n].block_name;
		struct1[1] = Tunable_Parameters[n].param_name;
		struct1[2] =  (int)Tunable_Parameters[n].n_rows;
		struct1[3] =  (int)Tunable_Parameters[n].n_cols;
		XmlRpcValue struct2;
		struct2.setSize((int)(Tunable_Parameters[n].n_rows*Tunable_Parameters[n].n_cols));
		for (unsigned int nr = 0; nr < Tunable_Parameters[n].n_rows; nr++) {
				for (unsigned int nc = 0; nc < Tunable_Parameters[n].n_cols; nc++) {
					struct2[nr*Tunable_Parameters[n].n_cols+nc] = Tunable_Parameters[n].data_value[nr*Tunable_Parameters[n].n_cols+nc];
				}
		}
		struct1[5] = struct2;
		Param_XML[n+1]=struct1;
	}
	result = Param_XML;
	
	if(IS_DEBUG) { std::cout << ">> End of Get_Param <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Get Target parameters"); }

} get_param(&s);    // This constructor registers the method with the server


/** \class Send_Param
* \brief Update new parameters on the connected target via XMLRPC.
* 
* This method is available as web service. It allows you to update new parameters
* on a connected target. If the specified target is not connected to the 
* server, this function return an error.
* @param params it has to contain vector, the first element has to be the task name.
* The remaining elements contain all the parameters of the target. So you have 
* to provided all the parameters not only the modified one. Each 
* element contains a parameters. Each parameter is an XmlRpcValue formed by
* 4+parameter dimension elements.The first is the block name, the second is the
* parameter name, the third is the number of rows the fourth is the number of
* columns, the last (number of rows * number of column) values contain the 
* numerical values of the parameters.
* @param result This variable contains the result of the execution of this 
* function. It is a vector of XmlRpcValue element of size number of parameters 
* +1. The first element contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
* 
* @return see parameter result. 
*/
class Send_Param : public XmlRpcServerMethod
{
public:
Send_Param(XmlRpcServer* s) : XmlRpcServerMethod("Send_Param", s) {}

//The first element is IFTASK after the parameters
void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Send_Param <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()==0) {
		status["str"]="Number of parameters in calling SEND_PARAMS incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(Verbose)
		std::cout<<"Send_Params: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	int n_rows = 0;
	int n_cols = 0;
	int map_offset=0;
	if (Verbose) {
		printf("number of elements in params: %d\n",params.size()-1);
	}
	try{
		for(int i=0; i < params.size()-1;i++){
			XmlRpcValue temp,temp_value;
			temp=params[i+1];
			n_rows = (int)temp[2];
			n_cols = (int)temp[3];
			temp_value=temp[4];
			for(int k=0; k<n_rows*n_cols;k++){
					char temptune[6];
					char tempjava[6];
					sprintf(temptune,"%1.3e",Tunable_Parameters[i].data_value[k]);
					sprintf(tempjava,"%1.3e",(double)temp_value[k]);
					if (Verbose) {
						printf("value :%s\n",tempjava);
						printf("tuneb :%s\n",temptune);
					}
					if(strcmp(temptune,tempjava)!=0){
						Tunable_Parameters[i].data_value[k]=(double)temp_value[k];
						if (Verbose) {
							printf("param modificato\n");
						}
						rlg_update_parameters_cb( k, map_offset,(double)temp_value[k]);
					}
			}
				map_offset++;   //this counter selects the parameter index to modify in the Target;
		}
	} catch(XmlRpcException e){
		status["str"]="Error in updating parameters.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	status["str"]="Parameters on target updated.";
	status["state"]=(int)((Is_Target_Running*8+Is_Target_Connected*4+2));
	result=status;

	if(IS_DEBUG) { std::cout << ">> End of Send_Param <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Upload Client parameter vector"); }

} send_param(&s);    // This constructor registers the method with the server

/** \class Get_Signal_Structure
* \brief Get the structure of all the signals of the specified 
* target via XMLRPC.
* 
* This method is available as web service. It allows you to get the structure 
* of all the signals of the connected target. If the specified target is not 
* connected to the server, this function return an error.
* @param params it has to contain as element 0 the task name.
* @param result This variable contains the result of the execution of this 
* function. It is a vector of XmlRpcValue element of size number of parameters 
* +1. The first element contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running. The remaining elements contain the signals 
* structure. Each element contains the structure of a signal. Each signal structure 
* is an XmlRpcValue formed by 3 dimension element.The first "str" is the 
* signal name, the second is the number of traces, the third is the sample time.
* If an error occurred only a one value vector is returned with the first 
* specified element.
* 
* @return see parameter result. 
*/
class Get_Signal_Structure : public XmlRpcServerMethod
{
public:
Get_Signal_Structure(XmlRpcServer* s) : XmlRpcServerMethod("Get_Signal_Structure", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Get_Signal_Structure <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	XmlRpcValue Param_XML;
	XmlRpcValue struct1;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling GET_SIGNAL_STRUCTURE incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(Verbose)
		std::cout<<"Get_Signal_Structure: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}	
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	status["str"]="Signals structure ok.";
	status["state"]=(int)((Is_Target_Running*8+Is_Target_Connected*4+2));
	Param_XML[0]=status;
	for (int n = 0; n < Num_Scopes; n++) {
		struct1["str"] = Scopes[n].name;
		struct1["i4"] = Scopes[n].ntraces;
		struct1["d"] = Scopes[n].dt;
		Param_XML[n+1]=struct1;
	}
	result = Param_XML;
	if(IS_DEBUG) std::cout << ">> End of Get_Signal_Structure <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Get signal structure from Target"); }

} get_signal_structure(&s);    // This constructor registers the method with the server


class Get_Digital_Structure : public XmlRpcServerMethod
{
public:
Get_Digital_Structure(XmlRpcServer* s) : XmlRpcServerMethod("Get_Digital_Structure", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Get_Digital_Structure <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	XmlRpcValue Param_XML;
	XmlRpcValue struct1;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling GET_DIGITAL_STRUCTURE incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(Verbose)
		std::cout<<"Get_Digital_Structure: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}	
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	status["str"]="Digitals structure ok.";
	status["state"]=(int)((Is_Target_Running*8+Is_Target_Connected*4+2));
	Param_XML[0]=status;
	for (int n = 0; n < Num_Leds; n++) {
		struct1["str"] = Leds[n].name;
		struct1["i4"] = Leds[n].n_leds;
		struct1["d"] = Leds[n].dt;
		Param_XML[n+1]=struct1;
	}
	result = Param_XML;
	
	if(IS_DEBUG) { std::cout << ">> End of Get_Digital_Structure <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Get digital structure from Target"); }

} get_digital_structure(&s);    // This constructor registers the method with the server


/** \class Start_Data
* \brief Enable the specified signal to send its data over the net.
* 
* This method is available as web service. It allows you to enable a signal
* to send the data over the net using the open socket with the specified 
* decimation.
* @param params it has to contain two or three elements. The first is the session 
* id, the second the signal number and the third (optional) the decimation
* factor. If not specified the decimation factor is the prevoius one, by default
* it's 1 that means that there's no decimation.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
*  
* @return see parameter result. 
*/
class Start_Data : public XmlRpcServerMethod
{
public:
Start_Data(XmlRpcServer* s) : XmlRpcServerMethod("Start_Data", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Start_Data <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	//First parameter IFTASK, the second index and the third decimation
	
	if(!params.valid() || params.size()<2 || params.size()>3) {
		status["str"]="Number of parameters in calling START_DATA incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Start_Data: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}

	if(!params[1].getType() == 2) {
		status["str"]="Signal Number incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	unsigned int index= (int)params[1];
	if(params.size()==3) {
		if(!params[2].getType() == 2) {
			status["str"]="Decimation Number incorrect.";
			status["state"]=(int)1;
			result=status;
			return;
		}
		unsigned int dec = (int)params[2];
		if(Signals_Manager->Set_Dec(index,dec)==-1) {
			status["str"]="Error in setting decimation.";
			status["state"]=(int)1;
			result=status;
			return;
		}
	}
		
	
	try{
		sem_wait(&mutex);       //used to modify the number of viewable signals.
		int index= (int)params[1];
		if(index>=0 && index<Num_Scopes){
				Signals_Manager->Start_Signal(index);
				if (Verbose) {
					printf("Signal %d enabled\n",index);
				}
			status["str"]="Signal Enabled";
			status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
		}
		else {
			status["str"]="Signal Number not correct";
			status["state"] = (int)1; 
		}
		sem_post(&mutex);       //Critical section ended. Socket thread is unlocked.
	}catch(XmlRpcException e){
		std::cout << e.getMessage();
			sem_post(&mutex);
		status["str"]="Error in setting signal";
		status["state"] = (int)1; 
	}
	result=status;
	
	if(IS_DEBUG) { std::cout << ">> End of Start_Data <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Request signals selected by the client"); }

} start_data(&s);    // This constructor registers the method with the server


/** \class Stop_Data
* \brief Stop the data sending from the specified signal. 
* 
* This method is available as web service. It allows you to stop the data
* sending from a signal.
* @param params it has to contain two elements. The first is the task 
* name, the second the signal number.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
*  
* @return see parameter result. 
*/

class Stop_Data : public XmlRpcServerMethod
{
public:
Stop_Data(XmlRpcServer* s) : XmlRpcServerMethod("Stop_Data", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop_Data <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=2) {
		status["str"]="Number of parameters in calling STOP_DATA incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Stop_Data: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}

	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}

	if(!params[1].getType() == 2) {
		status["str"]="Signal Number incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	
	try{
		sem_wait(&mutex);       //used to modify the number of viewable signals.
		int index= (int)params[1];
		if(index>=0 && index<Num_Scopes){
				Signals_Manager->Stop_Signal(index);
				if (Verbose) {
					printf("Signal %d disabled\n",index);
				}
			status["str"]="Signal Disabled";
			status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
		}
		else {
			status["str"]="Signal Number not correct";
			status["state"] = (int)1; 
		}
		sem_post(&mutex);       //Critical section ended. Socket thread is unlocked.
	}catch(XmlRpcException e){
			std::cout << e.getMessage();
			sem_post(&mutex);
		status["str"]="Error in resetting signal";
		status["state"] = (int)1; 
	}
	result=status;

	if(IS_DEBUG) { std::cout << ">> End of Stop_Data <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Request signals selected by the client"); }

} stop_data(&s);    // This constructor registers the method with the server

/** \class Start_Digital
* \brief Enable the specified signal to send its data over the net.
* 
* This method is available as web service. It allows you to enable a signal
* to send the data over the net using the open socket with the specified 
* decimation.
* @param params it has to contain two or three elements. The first is the session 
* id, the second the signal number and the third (optional) the decimation
* factor. If not specified the decimation factor is the prevoius one, by default
* it's 1 that means that there's no decimation.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
*  
* @return see parameter result. 
*/
class Start_Digital : public XmlRpcServerMethod
{
public:
Start_Digital(XmlRpcServer* s) : XmlRpcServerMethod("Start_Digital", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Start_Digital <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	//First parameter IFTASK, the second index and the third trigger(O OFF - 1 ON default OFF)
	
	if(!params.valid() || params.size()<2 || params.size()>3) {
		status["str"]="Number of parameters in calling START_DATA incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Start_Digital: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}

	if(!params[1].getType() == 2) {
		status["str"]="Digital Number incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	unsigned int index= (int)params[1];
	if(params.size()==3) {
		if(!params[2].getType() == 2 || ((int)params[2]!=0 && (int)params[2]!=1)) {
			status["str"]="Trigger Number incorrect.";
			status["state"]=(int)1;
			result=status;
			return;
		}
		unsigned int trig = (int)params[2];
		if(!Digitals_Manager->Set_Trigger(index,trig)) {
			status["str"]="Error in setting trigger mode.";
			status["state"]=(int)1;
			result=status;
			return;
		}
	}
		
	
	try{
		sem_wait(&mutex);       //used to modify the number of viewable signals.
		int index= (int)params[1];
		if(index>=0 && index<Num_Scopes){
				Digitals_Manager->Start_Digital(index);
				if (Verbose) {
					printf("Digital %d enabled\n",index);
				}
			status["str"]="Digital Enabled";
			status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
		}
		else {
			status["str"]="Digital Number not correct";
			status["state"] = (int)1; 
		}
		sem_post(&mutex);       //Critical section ended. Socket thread is unlocked.
	}catch(XmlRpcException e){
		std::cout << e.getMessage();
			sem_post(&mutex);
		status["str"]="Error in setting Digital";
		status["state"] = (int)1; 
	}
	result=status;

	if(IS_DEBUG) { std::cout << ">> End of Start_Digital <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Request Digitals selected by the client"); }

} start_digital(&s);    // This constructor registers the method with the server


/** \class Stop_Digital
* \brief Stop the data sending from the specified digital. 
* 
* This method is available as web service. It allows you to stop the data
* sending from a digital.
* @param params it has to contain two elements. The first is the session id, 
* the second the digital number.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
*  
* @return see parameter result. 
*/

class Stop_Digital : public XmlRpcServerMethod
{
public:
Stop_Digital(XmlRpcServer* s) : XmlRpcServerMethod("Stop_Digital", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop_Digital <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=2) {
		status["str"]="Number of parameters in calling Stop_Digital incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Stop_Digital: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}

	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}

	if(!params[1].getType() == 2) {
		status["str"]="Digital Number incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	
	try{
		sem_wait(&mutex);       //used to modify the number of viewable signals.
		int index= (int)params[1];
		if(index>=0 && index<Num_Leds){
				Digitals_Manager->Stop_Digital(index);
				if (Verbose) {
					printf("Signal %d disabled\n",index);
				}
			status["str"]="Digital Disabled";
			status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
		}
		else {
			status["str"]="Digital Number not correct";
			status["state"] = (int)1; 
		}
		sem_post(&mutex);       //Critical section ended. Socket thread is unlocked.
	}catch(XmlRpcException e){
			std::cout << e.getMessage();
			sem_post(&mutex);
		status["str"]="Error in resetting Digital";
		status["state"] = (int)1; 
	}
	result=status;
	
	if(IS_DEBUG) { std::cout << ">> End of Stop_Digital <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Request digitals selected by the client"); }

} stop_digital(&s);    // This constructor registers the method with the server


/** \class Stop_All_Data
* \brief Stop all data transfer.
* 
* This method is available as web service. It allows you to stop the sending of 
* all signals to send the data over the net using the open socket without 
* decimation.
* @param params it has to contain as element 0 the task name.
* @param result This variable contains the result of the execution of this 
* function. It contains two values. The first "str" is an human readable
* string that explains what's happened. The second "state" is an integer value
* that contains the state of the system (target+server) and can be explained in 
* this way. The first four bits contains the useful information. The less one 
* indicates if an error occurred, the second shows if the target exists, the third
* let you know if the target is connected to the server and the last if the
* connected target is running.
*  
* @return see parameter result. 
*/

class Stop_All_Data : public XmlRpcServerMethod
{
public:
Stop_All_Data(XmlRpcServer* s) : XmlRpcServerMethod("Stop_All_Data", s) {}

void execute(XmlRpcValue& params, XmlRpcValue& result)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> Stop_All_Data <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	XmlRpcValue status;
	
	if(!params.valid() || params.size()!=1) {
		status["str"]="Number of parameters in calling STOP_ALL_DATA incorrect";
		status["state"]=(int)1;
		result=status;
		return;
	}
	
	if(Verbose)
		std::cout<<"Stop_All_Data: parametro->"<<params[0]<<"\t Valore corretto->"<<sIdSession<<"\n";
	if(std::string(params[0]).compare(sIdSession) ) {
		status["str"]="Session ID incorrect.";
		status["state"]=(int)1;
		result=status;
		return;
	}
	if(!Is_Target_Connected){
		status["str"]="Target not connected";
		status["state"] = (int)0; 
		result=status;
		return;
	}

	if(!Is_There_A_Target(TargetName)) {
		status["str"]="No "+TargetName+" target or bad interface task identifier";
		status["state"]=(int)0;
		result=status;
		return;
	}
	
	try{
		sem_wait(&mutex);       //used to modify the number of viewable signals.
		Signals_Manager->ResetAll();
		Digitals_Manager->ResetAll();
		status["str"]="All Signals Disabled";
		status["state"] = (int)((Is_Target_Running*8+Is_Target_Connected*4+2)); 
		sem_post(&mutex);       //Critical section ended. Socket thread is unlocked.
	}catch(XmlRpcException e){
			sem_post(&mutex);
		status["str"]="Error in resetting signals";
		status["state"] = (int)1; 
	}
	result=status;
	
	if(IS_DEBUG) { std::cout << ">> End of Stop_All_Data <<" << std::endl; std::fflush(stdout); }
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
}

std::string help() { return std::string("Reset all signals."); }

} stop_all_data(&s);    // This constructor registers the method with the server


/** @} */ // end of slave


/** \brief Load preferences. 
*/ 
void rlg_read_pref(void)
{
				Preferences.Target_Interface_Task_Name= "DEFAULT_TARGET_NAME";
				Preferences.Target_Scope_Mbx_ID = "DEFAULT_SCOPE_NAME";
				Preferences.Target_Log_Mbx_ID = "RTL";
				Preferences.Target_Led_Mbx_ID = "DEFAULT_DIGITAL_NAME";
				Preferences.Target_Meter_Mbx_ID = "RTM";
				Preferences.Target_Synch_Mbx_ID = "RTY";
}

/** \brief Stop the server.
* 
* This function stops the server execution in a safe way. It disconnects the
* target from the server, kills all thread and stops the external interface. * 
* 
*/ 
void rlg_quit_cb(void)
{
	if(IS_DEBUG) std::cout << std::endl << "--------------------------------------------------------------------------------" << std::endl;
	if(IS_DEBUG) std::cout << ">> rlg_quit_cb <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	if (Is_Target_Connected) {
		RT_RPC(Target_Interface_Task, DISCONNECT_FROM_TARGET, 0);
	}
	else {
		if(!client_connected && client_waiting) {
			Socket data_sock;
			data_sock.create();
			data_sock.connect("localhost",portSocket);
			client_connected=false;
			client_waiting=false;
			sending_sock.Close();
		}
	}
	
	//Shutdown the apps if the request comes in slave mode or from a stop_server or from an automatic target termination.
	if(bCompleteShutdown || bSlave)
	{
		End_App = 1;
		std::fflush(stdout);
		for (int n = 0; n < Num_Scopes; n++) {
			//These threads will stop because of End_App=1
			pthread_join(Get_Scope_Data_Thread[n], NULL);
		}
		for (int n = 0; n < Num_Leds; n++) {
			//These threads will stop because of End_App=1
			if(IS_DEBUG) std::cout << " " << n << ", "; 		
			std::fflush(stdout);
			pthread_join(Get_Led_Data_Thread[n], NULL);
		}
		
		if(Signals_Manager!=NULL)
				Signals_Manager->ResetAll();
		if(Digitals_Manager!=NULL)
				Digitals_Manager->ResetAll();
	//    pthread_join(Send_Data_Target_Thread,NULL);
	
		listen_sock.Close();
		rt_send(Target_Interface_Task, CLOSE);
		pthread_join(Target_Interface_Thread, NULL);
	
		End_All = 1;
	
		rt_task_delete(RLG_Main_Task);
		s.shutdown();	//Stop of the XMLRPC Server
	}
	
	//Detaches the session from the master server 
	if(bSlave)
	{
		XmlRpcClient c("localhost", MasterXMLPort);
		XmlRpcValue noArgs, result;
		noArgs[0]=MasterServerName.c_str();
		noArgs[1]=sIdSession.c_str();
		try {
			if (c.execute("Close_Session", noArgs, result))
				if(IS_DEBUG) std::cout << result << std::endl; 		
			else {
				if(IS_DEBUG) std::cout << "Error!" << std::endl; 		
			}
		}catch (XmlRpcException ex) {
			if(IS_DEBUG) std::cout << "Error in connection with the master server.." << std::endl; 		
		}
	}

	if(IS_DEBUG) std::cout << ">> End of rlg_quit_cb <<" << std::endl;
	if(IS_DEBUG) std::cout << "--------------------------------------------------------------------------------" << std::endl;
	return;
}

/*
Update more than one parameter at the time.
If the checkbox batch_download is disabled, task_interface is called with code UPDATE_PARAM,
otherwise there is no update and the new values are pushed into Batch_Parameters.
*/
void rlg_update_parameters_cb(int ind, int map_offset, double value)
{
	//value che rappresenterebbe il nuovo valore non viene utilizzato in
	// quanto tale valore ? gi? presente in Tunable_Parameters (vi ? stato messo
	// all'atto della chiamata XMLRPC). Quindi l'azione seguente riguarda
	// solamente l'esecuzione di UPDATE_PARAM in rt_task_interface.
	RT_RPC(Target_Interface_Task, (ind << 20) | (map_offset << 4) | UPDATE_PARAM, 0);
}


/** \brief Get data from the mailboxes and send them to the socket. 
* 
* This function represents a thread. Each scope has its own thread.
* @param  
*/ 
static void *rt_get_signal_data(void *arg)
{
	bool IS_DEBUG_FUNCTION = false;
	RT_TASK *GetScopeDataTask;
	MBX *GetScopeDataMbx;
	char GetScopeDataMbxName[7];
	int MsgData = 0, MsgLen, MaxMsgLen, TracesBytes, MsgWait;
	float MsgBuf[MAX_MSG_LEN/sizeof(float)];
	int n, nn, jl;
	int index = ((Args_T *)arg)->index;
	char *mbx_id = strdup(((Args_T *)arg)->mbx_id);
	unsigned short Number_Samples=0;
	unsigned short Prefix_Number=0;
	unsigned short Actual_Dec=Signals_Manager->Get_Dec(index);
	int error_socket=0;
	if(IS_DEBUG) { std::cout << "\t\t rt_get_signal_data:: BEGIN - Scopes number " << index << std::endl; std::fflush(stdout); }

	rt_allow_nonroot_hrt();
	if (!(GetScopeDataTask = rt_task_init_schmod(get_an_id("HGS"), 99, 0, 0, SCHED_RR, 0xFF))) {
		printf("Cannot init Host GetScopeData Task\n");
		return (void *)1;
	}
	sprintf(GetScopeDataMbxName, "%s%d", mbx_id, index);
	if (!(GetScopeDataMbx = (MBX *)rt_get_adr(nam2num(GetScopeDataMbxName)))) {
		printf("Error in getting %s mailbox address\n", GetScopeDataMbxName);
		return (void *)1;
	}
	TracesBytes = (Scopes[index].ntraces + 1)*sizeof(float);
	//Per aver un multiplo intero non nullo di TracesBytes.
	MaxMsgLen = (MAX_MSG_LEN>TracesBytes)?(MAX_MSG_LEN/TracesBytes)*TracesBytes:TracesBytes;
	//In MsgLen si ha un multiplo intero di TracesBytes in ogni caso.
	//Se il tempo di campionamento e' molto piccolo allora si leggono piu' dati in un unico blocco.
	MsgLen = (((int)(TracesBytes*REFRESH_RATE*(1./Scopes[index].dt)))/TracesBytes)*TracesBytes;
	if(Verbose)
		std::cout<<"MsgLen:"<<MsgLen<<" dt: "<<Scopes[index].dt<<"\n";
	//Si legge almeno un dato.
	if (MsgLen < TracesBytes) MsgLen = TracesBytes;
	//Se il tempo di campionamento e' troppo piccolo allora si legge la massima capienza della mbx
	// compatibile con le dimensioni di TracesBytes.
	if (MsgLen > MaxMsgLen) MsgLen = MaxMsgLen;
	MsgData = MsgLen/TracesBytes;
	MsgWait = (int)(((int)((MAX_MSG_SIZE-MsgLen)/TracesBytes))*Scopes[index].dt*1000);
	MsgWait=(MsgWait>WAIT_MIN)?MsgWait:WAIT_MIN;
	MsgWait=(MsgWait<WAIT_MAX)?MsgWait:WAIT_MAX;
	if(Verbose)
		std::cout<<"MsgWait:"<<MsgWait<<"\n";
	

	//Manda 0 al thread principale, che arriva alla receive dopo la creazione dei thread
	rt_send(Target_Interface_Task, 0);
	//Per spiegazioni su mlockall: http://docs.hp.com/en/B9106-90009/mlockall.2.html
	//Praticamente fa in modo di rimanere in memoria e non essere messo in memoria
	// virtuale.
//    mlockall(MCL_CURRENT | MCL_FUTURE);

	if(IS_DEBUG) { std::cout << "\t\t rt_get_signal_data:: ready - Scopes number " << index << std::endl; std::fflush(stdout); }

	while (error_socket<11) {
		if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: End Application? "<< (End_App?"Si - ":"No - ") << "Target connesso? "<< (Is_Target_Connected?"Si.":"No.") << std::endl;  std::fflush(stdout); }
		if (End_App || !Is_Target_Connected) break;
		while (rt_mbx_receive_if(GetScopeDataMbx, &MsgBuf, MsgLen)) {
			if (End_App || !Is_Target_Connected) {
				if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Deleting - Scopes number " << index << std::endl; std::fflush(stdout); }
				rt_task_delete(GetScopeDataTask);
				return 0;
			}
			if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Looking for target " << TargetName <<" - Scopes number " << index << std::endl; std::fflush(stdout); }
			if(!Is_There_A_Target(TargetName)) {
				if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Target not found sending STOP_TARGET - Scopes number " << index << std::endl; std::fflush(stdout); }
				if(sem_wait(&mutex_close)==-1)
					std::cout<<"Errore (1) nello scope " << index;
				if(!End_App)
					RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
				if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Target not found sent STOP_TARGET- Scopes number " << index << std::endl; std::fflush(stdout); }
				sem_post(&mutex_close);
			} else 
				if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Target found - Scopes number " << index << std::endl; std::fflush(stdout); }
				msleep(MsgWait);
		}
		//Starts sending data to socket
		if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Signal activated? "<< (Signals_Manager->Get_Signal(index)?"Si":"No") <<" - Scopes number " << index << std::endl; std::fflush(stdout); }
		if (Signals_Manager->Get_Signal(index)) {
			if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: Sending data... - Scopes number " << index << std::endl; std::fflush(stdout); }
				jl = 0;
				for (n = 0; n < MsgData; n++) {
					Number_Samples++;
					if(Number_Samples%Signals_Manager->Get_Dec(index)==0)
					{
						Number_Samples=0;
						char temp[20];
						//Metto l'indice del scope davanti ai dati.
						sprintf(temp,"%d",index);
						std::string data(temp);
						Prefix_Number++;
						if(Prefix_Number==0)
							Prefix_Number=1;
						if(Actual_Dec!=Signals_Manager->Get_Dec(index)) {
							Prefix_Number=0;
							Actual_Dec=Signals_Manager->Get_Dec(index);
						}
						sprintf(temp," %d",Prefix_Number);
						data=data + temp;
						//Il jl++ e nn=1 all'inizio del for mi servono per saltare il tempo
						for (nn = 1, jl++; nn < Scopes[index].ntraces + 1; nn++) {
							sprintf(temp," %e",MsgBuf[jl++]);
							data=data + temp;
						}
						if(sem_wait(&mutex_sock)==-1)
							std::cout<<"Errore nello scope " << index;
						try{
							sending_sock << data + "\n";
						}
						catch (SocketException e) {
							//Non scrivo nessun messaggio d'errore perche' pu? darsi che non si 
							// riesca pi? a scrivere sul socket perche' il client non fa le receive
							if(Verbose)
								std::cout<<e.description()<<"\t"<<data<<"\n";
						}					
						sem_post(&mutex_sock);
					} else
						jl+=(Scopes[index].ntraces + 1);
				}
			if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_signal_data:: End of Sending data - Scopes number " << index << std::endl; std::fflush(stdout); }
		}
		//Ends sending data to socket
	}
	if(IS_DEBUG) { std::cout << "\t\t rt_get_signal_data:: end - Scopes number " << index << std::endl; std::fflush(stdout); }

	if (Verbose)
		printf("Deleting scope thread number...%d\n", index);
	rt_task_delete(GetScopeDataTask);
	return 0;
}

/** \brief Get data from the mailboxes and send them to the socket. 
* 
* This function represents a thread. Each Led has its own thread.
* @param  
*/ 
static void *rt_get_digital_data(void *arg)
{
	bool IS_DEBUG_FUNCTION = true;

	RT_TASK *GetLedDataTask;
	MBX *GetLedDataMbx;
	char GetLedDataMbxName[7];
	int MsgData = 0, MsgLen, MaxMsgLen, DataBytes, MsgWait;
	unsigned int MsgBuf[MAX_MSG_LEN/sizeof(unsigned int)];
	int n;
	int index = ((Args_T *)arg)->index;
	char *mbx_id = strdup(((Args_T *)arg)->mbx_id);

	unsigned int PreviousData=-1;
	int error_socket=0;
	if(IS_DEBUG) { std::cout << "\t\t rt_get_digital_data:: BEGIN - Led number " << index << std::endl; std::fflush(stdout); }

	rt_allow_nonroot_hrt();
	if (!(GetLedDataTask = rt_task_init_schmod(get_an_id("HGE"), 99, 0, 0, SCHED_RR, 0xFF))) {
		printf("Cannot init Host GetLedData Task\n");
		return (void *)1;
	}
	sprintf(GetLedDataMbxName, "%s%d", mbx_id, index);
	if (!(GetLedDataMbx = (MBX *)rt_get_adr(nam2num(GetLedDataMbxName)))) {
		printf("Error in getting %s mailbox address\n", GetLedDataMbxName);
		exit(1);
	}
	DataBytes = sizeof(unsigned int);
	MaxMsgLen = (MAX_MSG_LEN/DataBytes)*DataBytes;
	MsgLen = (((int)(DataBytes*REFRESH_RATE*(1./Leds[index].dt)))/DataBytes)*DataBytes;
	if (MsgLen < DataBytes) MsgLen = DataBytes;
	if (MsgLen > MaxMsgLen) MsgLen = MaxMsgLen;
	MsgData = MsgLen/DataBytes;

	MsgWait = 10;
	MsgWait=(MsgWait>WAIT_MIN)?MsgWait:WAIT_MIN;
	MsgWait=(MsgWait<WAIT_MAX)?MsgWait:WAIT_MAX;
	if(Verbose)
		std::cout<<"MsgWait:"<<MsgWait<<"\n";

	//Manda 0 al thread principale, che arriva alla receive dopo la creazione dei thread
	rt_send(Target_Interface_Task, 0);
	//Per spiegazioni su mlockall: http://docs.hp.com/en/B9106-90009/mlockall.2.html
	//Praticamente fa in modo di rimanere in memoria e non essere messo in memoria
	// virtuale.
//    mlockall(MCL_CURRENT | MCL_FUTURE);
// 	if(IS_DEBUG) { std::cout << "\t\t rt_get_digital_data:: ready - Led number " << index << std::endl; std::fflush(stdout); }

	while (error_socket<11) {
		if (End_App || !Is_Target_Connected) break;
		while (rt_mbx_receive_if(GetLedDataMbx, &MsgBuf, MsgLen)) {
			if (End_App || !Is_Target_Connected) {
				rt_task_delete(GetLedDataTask);
				return 0;
			}
			if(!Is_There_A_Target(TargetName)) {
				if(sem_wait(&mutex_close)==-1)
					std::cout<<"Errore (1) nel led " << index;
				if(!End_App)
					RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
				sem_post(&mutex_close);
			} else 
				msleep(MsgWait);
		}


		//Starts sending data to socket
		if (Digitals_Manager->Get_Digital(index)) {
				for (n = 0; n < MsgData; n++) {
					if(!Digitals_Manager->Get_Trigger(index) || PreviousData != MsgBuf[n])
					{
						char temp[20];
						//Metto l'indice del digital davanti ai dati.
						sprintf(temp,"-%d",index+1);
						std::string data(temp);
						if(PreviousData != MsgBuf[n])
							PreviousData = MsgBuf[n];
						sprintf(temp," %d",MsgBuf[n]); //MsbBuf[n] contiene la maschera di attivazione dei led.
						data=data + temp;
						if(sem_wait(&mutex_sock)==-1)
							std::cout<<"Errore nel Led " << index;
						try{
							sending_sock << data + "\n";
							if(IS_DEBUG_FUNCTION) { std::cout << "\t\t rt_get_digital_data:: Data: " << data << std::endl; std::fflush(stdout); }
						}
						catch (SocketException e) {
							//Non scrivo nessun messaggio d'errore perche' pu? darsi che non si 
							// riesca pi? a scrivere sul socket perche' il client non fa le receive
							if(Verbose)
								std::cout<<e.description()<<"\t"<<data<<"\n";
						}					
						sem_post(&mutex_sock);
					}
				}
		}
		//Ends sending data to socket
	}
	if(IS_DEBUG) { std::cout << "\t\t rt_get_digital_data:: end - Led number " << index << std::endl; std::fflush(stdout); }

	if (Verbose)
		printf("Deleting led thread number...%d\n", index);
	
	rt_task_delete(GetLedDataTask);
	return 0;
}

/** \brief Wait for incoming request to enable data transfer. 
* 
* @param 
*/ 
void *rt_send_data_target(void *arg)
{
	try
	{
		client_connected=false;
		client_waiting=true;
		listen_sock.accept ( sending_sock );
		client_waiting=false;
		client_connected=true;
		sem_post(&mutex_sock);
	}
	catch ( SocketException& e )
	{
		std::cout << "Exception was caught:" << e.description() << "\nExiting.\n";
	}
	pthread_exit(NULL);
	return 0;
}

/** \brief Load parameters from the target to the server. 
* 
* @param 
*/ 
double get_parameter(Target_Parameters_T p, int nr, int nc, int *val_idx)
{
	switch (p.data_class) {
		case rt_SCALAR:
				*val_idx = 0;
				return (p.data_value[0]);
		case rt_VECTOR:
				*val_idx = nc;
				return (p.data_value[nc]);
		case rt_MATRIX_ROW_MAJOR:
				*val_idx = nr*p.n_cols+nc;
				return (p.data_value[nr*p.n_cols+nc]);
		case rt_MATRIX_COL_MAJOR:
				*val_idx = nc*p.n_rows+nr;
				return (p.data_value[nc*p.n_rows+nr]);
		default:
				return (0.0);
	}
}

/** \brief Load preliminary information about parameters on connection.
* 
* This is used in Target_Interface_Task during a (CONNECT_TO_TARGET) to get the number of tunable parameters
* and define the struct Tunable_Parameters.  
* @param 
*/
static int get_parameters_info(RT_TASK *task)
{
	//'c' rappresenta il carattere da spedire al task (target) per sapere se
	// e' in esecuzione. Setta Is_Target_Running
	unsigned int req = 'c';
	//'i' rappresenta il carattere da spedire al target per ottenere le info
	// per Tunable_Parameters[0]
	char c_req = 'i';
	int blk_index = 0;
	int n_params = 0;

	//La funzione RT_rpc serve per spedire un messaggio req al task ed ottenere
	// la risposta in Is_Target_Runnig. Il mittente viene bloccato fino a 
	// quando non riceve risposta.
	//
	// QUI SI DEVE CONTROLLARE SE IL TARGET ESISTE!!!
	// non importa in quanto il controllo viene fatto in rt_target_interface
	// nella CONNECT_TO_TARGET.
	//
	rt_rpc(task, req, &Is_Target_Running);
	n_params = Is_Target_Running & 0xffff;
	Is_Target_Running >>= 16;
	if (n_params > 0) Tunable_Parameters = new Target_Parameters_T [n_params];
	else               Tunable_Parameters = new Target_Parameters_T [1];

	//La funzinoe RT_rpcx spedisce a task smsg (Send Message) della dimensione 
	// di ssize e aspetta di ricevere dal task il messaggio rmsg (Receive
	// Message) della dimenzione rsize (gli ultimi due campi vengono riempiti
	// dal destinatartio.
	//Nel caso specifico riceve il primo elemento di Tunable_Parameters, che viene
	// utlizzato solo per ottenere il nome del modello. Poi dopo nel cicli lo
	// riprende nuovamente.
	//static inline RT_TASK *RT_rpcx(unsigned long node, int port, RT_TASK *task, 
	// 								void *smsg, void *rmsg, int ssize, int rsize)
	rt_rpcx(task, &c_req, &Tunable_Parameters[0], sizeof(char), sizeof(Target_Parameters_T));
	RLG_Target_Name = strdup(Tunable_Parameters[0].model_name);

	for (int n = 0; n < n_params; n++) {
		//Come sopra, ma riceve tutti i singoli elementi del vettore Tunable_Parameters
		rt_rpcx(task, &c_req, &Tunable_Parameters[n], sizeof(char), sizeof(Target_Parameters_T));
		//Conta il numero di Tunable_Blocks che rappresenta il numero di blocchi che
		// hanno lo stesso nome.
		if (n > 0) {
				if (strcmp(Tunable_Parameters[n-1].block_name, Tunable_Parameters[n].block_name)) {
					Num_Tunable_Blocks++;
				}
		} else {
				Num_Tunable_Blocks = 1;
		}
	}
	//Crea i Tunable_Blocks
	if (Num_Tunable_Blocks > 0) Tunable_Blocks = new Target_Blocks_T [Num_Tunable_Blocks];
	blk_index = 0;
	for (int n = 0; n < n_params; n++) {
		if (n > 0) {
			//Per ogni coppia di parametri che sono nello stesso blocco viene creato
			// un blocco con il nome del blocco + un progressivo finale.
			// E viene settato l'offset.
				if (strcmp(Tunable_Parameters[n-1].block_name, Tunable_Parameters[n].block_name)) {
					//Si cambia blocco.
					blk_index++;
					strncpy(Tunable_Blocks[blk_index].name, Tunable_Parameters[n].block_name + strlen(Tunable_Parameters[0].model_name) + 1, MAX_NAMES_SIZE);
					Tunable_Blocks[blk_index].offset = n;
				}
		} else {
			//Questa funzione copia una parte di una stringa.
				strncpy(Tunable_Blocks[0].name, Tunable_Parameters[0].block_name + strlen(Tunable_Parameters[0].model_name) + 1, MAX_NAMES_SIZE);
				Tunable_Blocks[0].offset = 0;
		}
	}

	return n_params;
	//Dopo questa viene eseguita immediatamente get_scope_blocks_info
}


static void upload_parameters_info(/*long port,*/ RT_TASK *task)
{
	unsigned int req = 'g';
	char c_req = 'i';
	int n;

	if(Is_There_A_Target(TargetName)) {
		rt_rpc(task, req, &Is_Target_Running);
		if (Verbose) {
			printf("Upload parameters...\n");
		}
		for (n = 0; n < Num_Tunable_Parameters; n++) {
			rt_rpcx(task, &c_req, &Tunable_Parameters[n], sizeof(char), sizeof(Target_Parameters_T));
		}
	}
	else {
		RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
		if (Verbose)
				printf("Target not found in upload_parameters.\n");
	}
}


/** \brief Load preliminary information about scopes on connection.
* 
* @param 
*/
static int get_scope_blocks_info(/*long port,*/ RT_TASK *task, const char *mbx_id)
{
	int n_scopes = 0;
	int req = -1, msg;

	//Conta il numero di Scopes presenti cercando di collegarsi alle Mailbox
	// relative agli scopes, quando avviene un errore, a quel passo si ottiene
	// il numero di scopes presenti.
	for (int n = 0; n < MAX_RTAI_SCOPES; n++) {
		char mbx_name[7];
		//Il nome di una mailbox e' composto da mbx_id(nome passato alla
		// funzione) + un numero progressivo degli scopes.
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
				n_scopes = n;
				break;
		}
	}
	if (n_scopes > 0) Scopes = new Target_Scopes_T [n_scopes];
	for (int n = 0; n < n_scopes; n++) {
		char scope_name[MAX_NAMES_SIZE];
		Scopes[n].visible = false;
		//Per capire il motivo per cui vengono messi in quest'ordine andare a
		// vedere in rtmain.c alle righe 1106-1112
		rt_rpcx(task, &n, &Scopes[n].ntraces, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &scope_name, sizeof(int), sizeof(scope_name));
		strncpy(Scopes[n].name, scope_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &Scopes[n].dt, sizeof(int), sizeof(float));
	}
	//In rtmain nel case 'c' c'e' un while(1) quando prende gli scopes e quando
	// gli passi un numero negativo come indice dello scope (nella rpcx sotto
	// in &req) esce da quel ciclo.
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_scopes;
	//Dopo questa viene eseguita immediatamente get_log_blocks_info
	
}

static int get_log_blocks_info(/*long port, */RT_TASK *task, const char *mbx_id)
{
	//n_logs represents the number of logs in the target
	int n_logs = 0;
	int req = -1, msg;
	
	//Counts the number of logs checking the presence of the corresponding mbx
	for (int n = 0; n < MAX_RTAI_LOGS; n++) {
		char mbx_name[7];
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
				n_logs = n;
				break;
		}
	}
	
	if (n_logs > 0) Logs = new Target_Logs_T[n_logs];
	for (int n = 0; n < n_logs; n++) {
		char log_name[MAX_NAMES_SIZE];
		rt_rpcx(task, &n, &Logs[n].nrow, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &Logs[n].ncol, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &log_name, sizeof(int), sizeof(log_name));
		strncpy(Logs[n].name, log_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &Logs[n].dt, sizeof(int), sizeof(float));
	}
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_logs;
	//Dopo questa viene eseguita immediatamente get_alog_blocks_info
}

static int get_alog_blocks_info(/*long port, */RT_TASK *task, const char *mbx_id)            //added il 4/5/2005
{
	int n_alogs = 0;
	int req = -1, msg;
	for (int n = 0; n < MAX_RTAI_LOGS; n++) {
		char mbx_name[7];
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
			n_alogs = n;
			break;
		}
	}
	if (n_alogs > 0) ALogs = new Target_ALogs_T [n_alogs];
	for (int n = 0; n < n_alogs; n++) {
		char alog_name[MAX_NAMES_SIZE];
		rt_rpcx(task, &n, &ALogs[n].nrow, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &ALogs[n].ncol, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &alog_name, sizeof(int), sizeof(alog_name));
		strncpy(ALogs[n].name, alog_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &ALogs[n].dt, sizeof(int), sizeof(float));
	}
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_alogs;
	//Dopo questa viene eseguita immediatamente get_led_blocks_info
}


static int get_led_blocks_info(/*long port,*/ RT_TASK *task, const char *mbx_id)
{
	int n_leds = 0;
	int req = -1, msg;

	for (int n = 0; n < MAX_RTAI_LEDS; n++) {
		char mbx_name[7];
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
				n_leds = n;
				break;
		}
	}
	if (n_leds > 0) Leds = new Target_Leds_T [n_leds];
	for (int n = 0; n < n_leds; n++) {
		char led_name[MAX_NAMES_SIZE];
		Leds[n].visible = false;
		rt_rpcx(task, &n, &Leds[n].n_leds, sizeof(int), sizeof(int));
		rt_rpcx(task, &n, &led_name, sizeof(int), sizeof(led_name));
		strncpy(Leds[n].name, led_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &Leds[n].dt, sizeof(int), sizeof(float));
	}
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_leds;
	//Dopo questa viene eseguita immediatamente get_meter_blocks_info
}

static int get_meter_blocks_info(/*long port,*/ RT_TASK *task, const char *mbx_id)
{
	int n_meters = 0;
	int req = -1, msg;

	for (int n = 0; n < MAX_RTAI_METERS; n++) {
		char mbx_name[7];
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
				n_meters = n;
				break;
		}
	}
	if (n_meters > 0) Meters = new Target_Meters_T [n_meters];
	for (int n = 0; n < n_meters; n++) {
		char meter_name[MAX_NAMES_SIZE];
		Meters[n].visible = false;
		rt_rpcx(task, &n, &meter_name, sizeof(int), sizeof(meter_name));
		strncpy(Meters[n].name, meter_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &Meters[n].dt, sizeof(int), sizeof(float));
	}
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_meters;
	//Dopo questa viene eseguita immediatamente get_synch_blocks_info
}

static int get_synch_blocks_info(/*long port,*/ RT_TASK *task, const char *mbx_id)
{
	int n_synchs = 0;
	int req = -1, msg;

	for (int n = 0; n < MAX_RTAI_SYNCHS; n++) {
		char mbx_name[7];
		sprintf(mbx_name, "%s%d", mbx_id, n);
		if (!rt_get_adr(nam2num(mbx_name))) {
				n_synchs = n;
				break;
		}
	}
	if (n_synchs > 0) Synchs = new Target_Synchs_T [n_synchs];
	for (int n = 0; n < n_synchs; n++) {
		char synch_name[MAX_NAMES_SIZE];
		Synchs[n].visible = false;
		rt_rpcx(task, &n, &synch_name, sizeof(int), sizeof(synch_name));
		strncpy(Synchs[n].name, synch_name, MAX_NAMES_SIZE);
		rt_rpcx(task, &n, &Synchs[n].dt, sizeof(int), sizeof(float));
	}
	rt_rpcx(task, &req, &msg, sizeof(int), sizeof(int));

	return n_synchs;
	//Fine
}
//Bug in RTAI le delete danno dei problemi.
void delete_all_pointers()
{
	if(Verbose)
		std::cout<<"Signals_Manager: "<<Signals_Manager<<"\n";
	if(Signals_Manager) {
		try{delete Signals_Manager;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Signals_Manager";}
	}
	if(Verbose)
		std::cout<<"Digitals_Manager: "<<Digitals_Manager<<"\n";
	if(Digitals_Manager) {
		try{delete Digitals_Manager;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Digitals_Manager";}
	}
	if(Verbose)
		std::cout<<"Tunable_Parameters: "<<Tunable_Parameters<<"\n";
	if(Tunable_Parameters) {
		try{delete[] Tunable_Parameters;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Tunable_Parameters";}
	}
	if(Verbose)
		std::cout<<"Tunable_Blocks: "<<Tunable_Blocks<<"\n";
	if(Tunable_Blocks) {
		try{delete[] Tunable_Blocks;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Tunable_Blocks";}
	}
	if(Verbose)
		std::cout<<"Scopes: "<<Scopes<<"\n";
	if(Scopes) {
		try{delete[] Scopes;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Scopes";}
	}
	if(Verbose)
		std::cout<<"Logs: "<<Logs<<"\n";
	if(Logs) {
		try{delete[] Logs;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Logs";}
	}
	if(Verbose)
		std::cout<<"Leds: "<<Leds<<"\n";
	if(Leds) {
		try{delete[] Leds;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Leds";}
	}
	if(Verbose)
		std::cout<<"Meters: "<<Meters<<"\n";
	if(Meters) {
		try{delete[] Meters;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Meters";}
	}
	if(Verbose)
		std::cout<<"Synchs: "<<Synchs<<"\n";
	if(Synchs) {
		try{delete[] Synchs;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Synchs";}
	}
	if(Verbose)
		std::cout<<"Get_Scope_Data_Thread: "<<Get_Scope_Data_Thread<<"\n";
	if(Get_Scope_Data_Thread) {
		try{delete Get_Scope_Data_Thread;}
		catch(...)
		{std::cout<<"Errore nell'eliminazione di Get_Scope_Data_Thread";}
	}
}


/** \brief Interface thread between the server and the target
* 
* @param 
*/
static void *rt_target_interface(void *args)
{
	unsigned int code, U_Request;
	RT_TASK *If_Task = NULL, *task;
	unsigned int msg;

	rt_allow_nonroot_hrt();
	if (!(Target_Interface_Task = rt_task_init_schmod(get_an_id("HTI2"), 98, 0, 0, SCHED_FIFO, 0xFF))) {
		printf("Cannot init Target_Interface_Task\n");
		exit(1);
	}
	rt_send(RLG_Main_Task, 0);      //my comment based on rtailab.c
	msleep(100);

	while (!End_App) {
		if (!(task = rt_receive(0, &code))) continue;
		if (Verbose) {
				printf("Received code %d from task %p\n", code, task);
		}
		//In quanto considera solamente i primi 4 bit di code, gli altri bit
		// contengono altre informazioni (vedi il caso di UPDATE_PARAM)
		switch (code & 0xf) {

				case CONNECT_TO_TARGET:
				if(Verbose)
					std::cout<<"Begin CONNECT \n";
					try{listen_sock.Reset();}
					catch (SocketException e) {std::cout<<"Error in connect ricreazione del socket\n";}
					if (Verbose)
						printf("Reading target settings\n");
					//void* rt_get_adr(unsigned long name)
					//Get an object address by its name.
					//rt_get_adr returns the address associated to name.
					//Returns: the address associated to name on success, 0 on failure 
					//Controlla se il target esiste controllando se il relativo nome e'
					// stato o meno registrato nel dominio RTAI.
					if (!(If_Task = (RT_TASK *)rt_get_adr(nam2num(TargetName.c_str())))) {
						if (Verbose) {
								printf("No target or bad interface task identifier\n");
						}
						RT_RETURN(task, CONNECT_TO_TARGET);
						break;
					}
					Num_Tunable_Parameters = get_parameters_info(/*Target_Port, */If_Task);
					if (Verbose) {
						printf("Target is running...%s\n", Is_Target_Running ? "yes" : "no");
						printf("Number of target tunable parameters...%d\n", Num_Tunable_Parameters);
						/*for (int n = 0; n < Num_Tunable_Parameters; n++) {
								printf("Block: %s\n", Tunable_Parameters[n].block_name);
								printf(" Parameter: %s\n", Tunable_Parameters[n].param_name);
								printf(" Number of rows: %d\n", Tunable_Parameters[n].n_rows);
								printf(" Number of cols: %d\n", Tunable_Parameters[n].n_cols);
								for (unsigned int nr = 0; nr < Tunable_Parameters[n].n_rows; nr++) {
									for (unsigned int nc = 0; nc < Tunable_Parameters[n].n_cols; nc++) {
										printf(" Value    : %f\n", Tunable_Parameters[n].data_value[nr*Tunable_Parameters[n].n_cols+nc]);
									}
								}
						}*/
					}
					Num_Scopes = get_scope_blocks_info(If_Task, ScopeMbxName.c_str());
					Signals_Manager=new Signal_Manager(Num_Scopes, Scopes);
					if (Verbose) {
						printf("Number of target real time scopes: %d\n", Num_Scopes);
						/*for (int n = 0; n < Num_Scopes; n++) {
								printf("Scope: %s\n", Scopes[n].name);
								printf(" Number of traces...%d\n", Scopes[n].ntraces);
								printf(" Sampling time...%f\n", Scopes[n].dt);
						}*/
					}
					get_log_blocks_info(If_Task, Preferences.Target_Log_Mbx_ID);
					if(iTargetInterfaceClass>=1)
						get_alog_blocks_info(If_Task, Preferences.Target_ALog_Mbx_ID);
					Num_Leds=get_led_blocks_info(If_Task, DigitalMbxName.c_str());
					Digitals_Manager=new Digital_Manager(Num_Leds, Leds);
					if (Verbose) {
						printf("Number of target real time leds: %d\n", Num_Leds);
					}
					get_meter_blocks_info(If_Task, Preferences.Target_Meter_Mbx_ID);
					get_synch_blocks_info(If_Task, Preferences.Target_Synch_Mbx_ID);
					Is_Target_Connected = 1;
					if (Verbose) {
						printf("Target %s is correctly connected\n", RLG_Target_Name);
					}
				if(Verbose)
					std::cout<<"Scopes threads initialization...";
					if (Num_Scopes > 0) Get_Scope_Data_Thread = new pthread_t [Num_Scopes];
					for (int n = 0; n < Num_Scopes; n++) {
						unsigned int msg;
						Args_T thr_args;
						thr_args.index = n;
						thr_args.mbx_id = strdup(ScopeMbxName.c_str());
//                    thr_args.mbx_id = strdup(Preferences.Target_Scope_Mbx_ID);
						thr_args.x = 500;
						thr_args.y = 290;
						thr_args.w = 250;
						thr_args.h = 250;
						pthread_create(&Get_Scope_Data_Thread[n], NULL, rt_get_signal_data, &thr_args);
						rt_receive(0, &msg);
					}
				if(Verbose) {
					std::cout<<"done.\n";
					std::cout<<"Digitals/Leds threads initialization...";
				}
				if (Num_Leds > 0) Get_Led_Data_Thread = new pthread_t [Num_Leds];
				for (int n = 0; n < Num_Leds; n++) {
					unsigned int msg;
					Args_T thr_args;
					thr_args.index = n;
					thr_args.mbx_id = strdup(DigitalMbxName.c_str());
					thr_args.x = 500;
					thr_args.y = 290;
					thr_args.w = 250;
					thr_args.h = 250;
					pthread_create(&Get_Led_Data_Thread[n], NULL, rt_get_digital_data, &thr_args);
					rt_receive(0, &msg);
				}
				if(Verbose) {
					std::cout<<"done.\n";
					std::cout<<"Client_waiting: "<<client_waiting<<".\n";
					std::cout<<"Socket thread initialization...";
					fflush(stdout);
				}
					pthread_create(&Send_Data_Target_Thread, NULL, rt_send_data_target, &msg);
					if(Verbose)
						std::cout<<"done.\n";
					RT_RETURN(task, CONNECT_TO_TARGET);
					if(Verbose)
						std::cout<<"end CONNECT\n";
					break;
				case DISCONNECT_FROM_TARGET:
					if(Verbose)
						std::cout<<"Begin DISCONNECT\n";
					if (Verbose)
						printf("Disconnecting from target %s\n", Tunable_Parameters[0].model_name);
					Is_Target_Connected = 0;
					if (Verbose)
					std::cout<<"Reset all signals and digitals...";
					Signals_Manager->ResetAll();
					Digitals_Manager->ResetAll();
					if (Verbose){
					std::cout<<"done.\n";
					std::cout<<"Client_connected: "<<client_connected<<"\n";
					}
					if(!client_connected && client_waiting) {
							if(Verbose)
						std::cout<<"Unblocking socket thread...";
					//Sbloccare il thread che e' fermo sulla accept.
					Socket data_sock;
					data_sock.create();
					data_sock.connect("localhost",portSocket);
					client_connected=false;
					client_waiting=false;
					}

					if(Verbose)
					std::cout<<"Stopping scopes threads...";
					for (int n = 0; n < Num_Scopes; n++) {
						//Blocca fino a quando il thread specificato non termina.
						pthread_join(Get_Scope_Data_Thread[n], NULL);
					}
					if(Verbose)
					std::cout<<"done.\n Stopping leds threads...";
					for (int n = 0; n < Num_Leds; n++) {
						//Blocca fino a quando il thread specificato non termina.
						pthread_join(Get_Led_Data_Thread[n], NULL);
					}
					if(Verbose)
						std::cout<<"done.\n";
					sem_wait(&mutex_sock);
					if(Verbose)
						std::cout<<"Sending data on socket stopped.\n";
					sending_sock.Close();
					pthread_join(Send_Data_Target_Thread, NULL);
					
					free(Tunable_Parameters);
					if (Verbose) {
						printf("Disconnected successfully.\n");
					}
					RT_RETURN(task, DISCONNECT_FROM_TARGET);
					if(Verbose)
						std::cout<<"End DISCONNECT.\n";
					break;

				case START_TARGET:
					if(Verbose)
						std::cout<<"Begin START...";
					if (!Is_Target_Running) {
						if (Verbose) {
								printf("Starting real time code...");
						}
						U_Request = 's';
					if(Is_There_A_Target(TargetName)) {
							rt_rpc(If_Task, U_Request, &Is_Target_Running);
							if (Verbose)
									printf("ok\n");
						}
						else {
						RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
							if (Verbose)
									printf("Target not found in START_TARGET.\n");
						}
					}
					RT_RETURN(task, START_TARGET);
					if(Verbose)
						std::cout<<"End START.\n";
					break;

				case STOP_TARGET:
				if(IS_DEBUG) { std::cout << "Begin STOP...\n"; std::fflush(stdout); }

					if (Is_Target_Running) {
						U_Request = 't';

						Is_Target_Connected = 0;
						if(IS_DEBUG) { std::cout << "Reset all signals and digitals..."; std::fflush(stdout); }
						Signals_Manager->ResetAll();
						Digitals_Manager->ResetAll();
						if(IS_DEBUG) { std::cout << "done.\n"; std::fflush(stdout); }

					if(IS_DEBUG) { std::cout << "Are Clients connected? "<< (client_connected?"Yes - ":"No - ") << "Are Clients waiting? "<< (client_waiting?"Yes.":"No."); std::fflush(stdout); }
						if(!client_connected && client_waiting) {
						if(IS_DEBUG) { std::cout << "Unblocking data socket thread..."; std::fflush(stdout); }
						Socket data_sock;
						data_sock.create();
						data_sock.connect("localhost",portSocket);
						client_connected=false;
						client_waiting=false;
						if(IS_DEBUG) { std::cout << "done.\n"; std::fflush(stdout); }
					}
					End_App = 1;

					sem_wait(&mutex_sock);
					if(IS_DEBUG) { std::cout << "Sending data on socket stopped.\n"; std::fflush(stdout); }
					sending_sock.Close();

					pthread_join(Send_Data_Target_Thread, NULL);					

					if(IS_DEBUG) { std::cout << "Stopping real time code..."; std::fflush(stdout); }

					if(Is_There_A_Target(TargetName))
						rt_rpc(If_Task, U_Request, &Is_Target_Running);

					if(IS_DEBUG) { std::cout << "done.\n"; std::fflush(stdout); }
					}
					RT_RETURN(task, STOP_TARGET);

				if(StopRequestFromTarget)
				{
					if(IS_DEBUG) { std::cout << "Termino i thread degli scopes.."; std::fflush(stdout); }
					for (int n = 0; n < Num_Scopes; n++) {
						if(IS_DEBUG) { std::cout << "Terminato lo scopes " << n << "." << std::endl; std::fflush(stdout); }
						pthread_join(Get_Scope_Data_Thread[n], NULL);
					}
					if(IS_DEBUG) { std::cout << "done.\n"; std::fflush(stdout); }
	
					if(IS_DEBUG) { std::cout << "Delete the real time task..."; std::fflush(stdout); }
					rt_task_delete(RLG_Main_Task);
					if(IS_DEBUG) { std::cout << "done." << std::endl;	std::fflush(stdout); }
					rt_task_delete(Target_Interface_Task);
					
					if(IS_DEBUG) std::cout << "Stop XMLRPC server..."; 		
					s.shutdown();	//Stop of the XMLRPC Server
					if(IS_DEBUG) std::cout << "done." << std::endl; 		
				
					
					//Detaches the session from the master server 
					if(bSlave)
					{
						XmlRpcClient c("localhost", MasterXMLPort);
						XmlRpcValue noArgs, result;
						noArgs[0]=MasterServerName.c_str();
						noArgs[1]=sIdSession.c_str();
						if (c.execute("Close_Session", noArgs, result))
							std::cout << result << "\n";
						else {
							std::cout << "Error!\n";
						}
					}
						
					if(IS_DEBUG) { std::cout << "End STOP.\n"; std::fflush(stdout); }
					exit(0);
						return 0;
				}
					break;

				case UPDATE_PARAM: {

					U_Request = 'p';
					//Attraverso gli shift e gli and si riprende i valori di map_offset
					// e k=Mat_Idx che erano stati inseriti all'interno di code.
					//shifta code di 4 bit cio? toglie UPDATE_PARAM e ne and con ffff=20bit a 1
					// quindi Map_Offset ? di al max 20 bit gli altri sono nulli
					int Map_Offset = (code >> 4) & 0xffff;
					//Poi toglie da code anche i 20 bit di map_offset e prende gli ultimi fff bit
					// in tutto un intero viene considerato di ffffffff bit (32 bit)
					int Mat_Idx = (code >> 20) & 0xfff;
					int Is_Param_Updated;
					double new_value = (double)Tunable_Parameters[Map_Offset].data_value[Mat_Idx];

				if(Is_There_A_Target(TargetName)) {
						rt_rpc(If_Task, U_Request, &Is_Target_Running);
						//Setta il parametro che si trova in Tunable_Parameters[Map_Offset].data_value[Mat_Idx]
						rt_rpcx(If_Task, &Map_Offset, &Is_Param_Updated, sizeof(int), sizeof(int));
						rt_rpcx(If_Task, &new_value, &Is_Param_Updated, sizeof(double), sizeof(int));
						rt_rpcx(If_Task, &Mat_Idx, &Is_Param_Updated, sizeof(int), sizeof(int));
						if (Verbose)
								printf("ok\n");
					}
					else {
					RT_RPC(Target_Interface_Task, STOP_TARGET, 0);
						if (Verbose)
								printf("Target not found in UPDATE_PARAM.\n");
					}
					RT_RETURN(task, UPDATE_PARAM);
					break;
				}
				
				case BATCH_DOWNLOAD: {
					break;
				}

				//set the vector Tunable_Parameters with the current Target parameters
				case GET_PARAMS: {
					upload_parameters_info(If_Task);
					/*if (Verbose) {
						printf("Target is running...%s\n", Is_Target_Running ? "yes" : "no");
						printf("Number of target tunable parameters...%d\n", Num_Tunable_Parameters);
						for (int n = 0; n < Num_Tunable_Parameters; n++) {
								printf("Block: %s\n", Tunable_Parameters[n].block_name);
								printf(" Parameter: %s\n", Tunable_Parameters[n].param_name);
								printf(" Number of rows: %d\n", Tunable_Parameters[n].n_rows);
								printf(" Number of cols: %d\n", Tunable_Parameters[n].n_cols);
								for (unsigned int nr = 0; nr < Tunable_Parameters[n].n_rows; nr++) {
									for (unsigned int nc = 0; nc < Tunable_Parameters[n].n_cols; nc++) {
										printf(" Value    : %f\n", Tunable_Parameters[n].data_value[nr*Tunable_Parameters[n].n_cols+nc]);
									}
								}
						}
					}*/
					RT_RETURN(task, GET_PARAMS);
					break;
				}
				case CLOSE:
					rt_task_delete(Target_Interface_Task);
					return 0;
				default:
					break;
		}
	}

	rt_task_delete(Target_Interface_Task);

	return 0;
}

struct option options[] = {
	{ "targetname",       	1, 0, 'n' },
	{ "scopename",       	1, 0, 'i' },
	{ "digitalname",       	1, 0, 'd' },
	{ "servername",       	1, 0, 'm' },
	{ "mastername",    	 	1, 0, 'M' },
	{ "masterport",    	 	1, 0, 'p' },
	{ "help",	      	 	0, 0, 'h' },
	{ "verbose",    			0, 0, 'v' },
	{ "version",    			0, 0, 'V' },
	{ "stop",   		 		0, 0, 's' },
	{ "portXML",   			1, 0, 'X' },
	{ "portSocket",    		1, 0, 'S' }
};



/** \brief Print help on standard output.
*/
void print_usage(void)
{
	fputs(
("\nUsage:  rtaixml [OPTIONS]\n"
"\n"
"OPTIONS:\n"
"  -M <Master Name>, --mastername <Master Name>\n"
"      set the master server name and the server as a slave in master-slave architecture, otherwise it works as a standalone rtaixml server.\n"
"  -m <Server_ID>, --servername <Server_ID>\n"
"      set the server name, [default=ServerId].\n"
"  -n <TaskName>, --taskname <TaskName>\n"
"      set the server name, [default=IFTASK].\n"
"  -i <ScopeName>, --scopename <ScopeName>\n"
"      set the server name, [default=RTS].\n"
"  -d <DigitalName>, --digitalname <DigitalName>\n"
"      set the server name, [default=RTE].\n"
"  -p <Master_XMLRPC_port>, --portXMLMaster <Master_XMLRPC_port>\n"
"      number of the XMLRPC server port\n"
"  -h, --help\n"
"      print usage\n"
"  -v, --verbose\n"
"      verbose output\n"
"  -V, --version\n"
"      print version\n"
"  -s, --stop\n"
"      stop server on localhost on the specified port\n"
"  -t, --target\n"
"      check the target to inspect RTAI-Lab version.\n"
"  -X <XMLRPC_port>, --portXML <XMLRPC_port>\n"
"      number of the XMLRPC server port, [default=29001].\n"
"  -S <Socket_port>, --portSocket <Socket_port>\n"
"      number of the Socket server port, [default=29002].\n"
"\n")
		,stderr);
	exit(0);
}

} // end namespace rtaixml_domain

using namespace rtaixml_domain;

int main(int argc, char **argv)
{
	char *lang_env;
	int c, option_index = 0;
	unsigned int msg;
	int test;
	XmlRpcValue oneArgs, result;
	int verbosityXMLRPC=0;
	RT_TASK *Verify_Server;
	bool toStop=false,bPort=false;
	
	//Inizialization of the random number generator.
	srand((unsigned)time(0)); 

	while (1) {
		c = getopt_long(argc, argv, "t:n:i:d:M:p:m:N:hvVsX:S:", options, &option_index);
		if (c == -1)
				break;
		switch (c) {
				case 't':
					iTargetInterfaceClass = atoi(optarg);
					std::cout<<"Target Interface Class set to "<<iTargetInterfaceClass<<"\n";
				break;
					case 'n':
					TargetName=std::string(optarg);
					std::cout<<"Target name set to "<<TargetName<<"\n";
				break;
				case 'i':
					ScopeMbxName=std::string(optarg);
					std::cout<<"Scope name set to "<<ScopeMbxName<<"\n";
					break;
				case 'd':
					DigitalMbxName=std::string(optarg);
					std::cout<<"Digital Scope name set to "<<DigitalMbxName<<"\n";
					break;
				case 'M':
					bSlave=true;
					MasterServerName=std::string(optarg);
					std::cout<<"Master server name set to "<<MasterServerName<<"\n";
					break;
				case 'p':
					if ((test = atoi(optarg)) && test>1024 && test<65537) {
						MasterXMLPort=test;
					}
					std::cout<<"Master XML Port set to "<<MasterXMLPort<<"\n";
					break;
				case 'm':
					sIdServer=std::string(optarg);
					std::cout<<"ID Server set to "<<sIdServer<<"\n";
					break;
				case 's':
						toStop=true;
						break;
				case 'v':
					Verbose = 1;
					verbosityXMLRPC=1;
					break;
				case 'V':
					fputs("rtaixml version " RTAIXML_VERSION "\n", stderr);
					exit(0);
				case 'X':
					if ((test = atoi(optarg)) && test>1024 && test<65536) {
						if(test==portSocket) {
							(portSocket==65536)?portSocket--:portSocket++;		
							std::cout << "\nThe given xmlrpc server port number is equal to the socket port one; \nmoving the socket port number to " << portSocket << " and setting the xmlrpc server port number to " << test <<"\n";
						}	
						portXML=test;
						if(!bPort)
							portSocket=portXML+1;
					}
					else
						std::cout << "The given port number is not correct, using default one.\n";
					printf("XMLRPC Port set to %d\n",portXML);
					if(!bPort)
						printf("Socket Port set to %d\n",portSocket);
						break;
				case 'S':
					if ((test = atoi(optarg)) && test>1024 && test<65537) {
						if(test==portXML) {
							(portXML==65536)?portXML--:portXML++;		
							std::cout << "\nThe given socket port number is equal to the xmlrpc server port one; \nmoving the xmlrpc server port number to " << portXML << " and setting the socket port number to " << test <<"\n";
						}	
						portSocket=test;
					}
					else
						std::cout << "The given port number is not correct, using default one.\n";
					printf("Socket Port set to %d\n",portSocket);
					bPort=true;
					break;
				case 'h':
					print_usage();
					exit(0);
				default:
					break;
		}
	}
	sIdSession=sIdServer;
	
	if(toStop)
	{
		printf("Stopping rtai-xml server on port %d...\n",portXML);
		oneArgs[0]=sIdSession.c_str();
		if((new XmlRpcClient("localhost", portXML))->execute("Stop_Server", oneArgs, result))
			std::cout << result << "\n";
		else
			std::cout << "Server not found on port " << portXML << ".\n";
		exit(0);
	}
		
	lang_env = getenv("LANG");
	setenv("LANG", "en_US", 1);
	rt_allow_nonroot_hrt();

/************** CHECK ANOTHER SERVER RUNNING ************************/
	int iCount;
	bool bTargetPresent = true;
	const int iMaxIterations = 100;
	Verify_Server = rt_task_init_schmod(nam2num("prova2"), 99, 0, 0, SCHED_FIFO, 0xFF); 

	for(iCount = 0, ServerName = get_random_string(5);
		rt_get_adr(nam2num(ServerName.c_str()))!=0 && iCount<iMaxIterations ; 
		iCount++,ServerName = get_random_string(5));
	rt_task_delete(Verify_Server);
	if(!bTargetPresent)
	{
		XmlRpcClient c("localhost", MasterXMLPort);
		XmlRpcValue noArgs, result;
		noArgs[0]=MasterServerName.c_str();
		noArgs[1]=sIdSession.c_str();
		if(IS_DEBUG) { std::cout << "Target not present! Closing Session on the master..."; std::fflush(stdout); }
		if (c.execute("Close_Session", noArgs, result)) {
			if(IS_DEBUG) { std::cout << "done." << std::endl; std::fflush(stdout); }
			if(IS_DEBUG) { std::cout << result << std::endl; std::fflush(stdout); }
		}
		else {
			if(IS_DEBUG) { std::cout << "ERROR!" << std::endl; std::fflush(stdout); }
		}
		return 0;
	}

	if (iCount==iMaxIterations) {
		printf("Cannot find a realtime name for the server.\n");
		exit(1);
	}
/*****************************************************************/
	
//Uncomment if you want to run rtaixml as a standalone daemon.
/*    if(daemon(1,Verbose)) {
		std::cout << "Error in creating server process, execution stopped.\n";
		exit(1);
	}*/
	if (!(RLG_Main_Task = rt_task_init_schmod(nam2num(ServerName.c_str()), 99, 0, 0, SCHED_FIFO, 0xFF))) {
		printf("Cannot init RTAI_LAB main task\n");
		exit(1);
	}
	rlg_read_pref();
	//Main thread creation for the Target interface
	pthread_create(&Target_Interface_Thread, NULL, rt_target_interface, NULL);
	rt_receive(0, &msg);
	msleep(100);
	sem_init(&mutex, 0, 1); /* initialize mutex to 1 - binary semaphore */
									/* second param = 0 - semaphore is local */

	if(sem_init(&mutex_sock, 0, 0)==-1)
			std::cout<<"errore nel mutex_sock"; 
	if(sem_init(&mutex_close, 0, 1)==-1)
			std::cout<<"errore nel mutex_sock"; 

	std::cout << "running....\n";

	XmlRpc::setVerbosity(verbosityXMLRPC);
	s.bindAndListen(portXML);
	// Enable introspection
	s.enableIntrospection(true);
	// wait for external connections
	listen_sock.Active(portSocket);
	s.work(-1.0);
	return 0;
}




