/** \file ServerSocket.cpp  
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "ServerSocket.h"
#include "SocketException.h"

namespace rtaixml_domain {

ServerSocket::ServerSocket ( int port )
{
	if ( ! Socket::create() )
	{
		throw SocketException ( "Could not create server socket." );
	}

	if ( ! Socket::bind ( port ) )
	{
		throw SocketException ( "Could not bind to port." );
	}

	if ( ! Socket::listen() )
	{
		throw SocketException ( "Could not listen to socket." );
	}

}

ServerSocket::~ServerSocket()
{
}


const ServerSocket& ServerSocket::operator << ( const std::string& s ) const
{
	if ( ! Socket::send ( s ) )
	{
		throw SocketException ( "Could not write to socket." );
	}

return *this;

}


const ServerSocket& ServerSocket::operator >> ( std::string& s ) const
{
	if ( ! Socket::recv ( s ) )
	{
		throw SocketException ( "Could not read from socket." );
	}

return *this;
}

const ServerSocket& ServerSocket::operator << ( const int& i ) const
{
	if ( ! Socket::send ( i ) )
	{
		throw SocketException ( "Could not write to socket." );
	}

return *this;

}


const ServerSocket& ServerSocket::operator >> (  int& i ) const
{
	if ( ! Socket::recv ( i ) )
	{
		throw SocketException ( "Could not read from socket." );
	}

return *this;
}



void ServerSocket::accept ( ServerSocket& sock )
{
	if ( ! Socket::accept ( sock ) )
	{
		throw SocketException ( "Could not accept socket." );
	}
}

void ServerSocket::Reset ( )
{
	if ( ! Socket::listen() )
	{
		throw SocketException ( "Could not listen to socket." );
	}
}

void ServerSocket::Close ( )
{
	Socket::reset();
}

void ServerSocket::Active ( int port )
{
	if ( ! Socket::create() )
	{
		throw SocketException ( "Could not create server socket." );
	}

	if ( ! Socket::bind ( port ) )
	{
		throw SocketException ( "Could not bind to port." );
	}

	if ( ! Socket::listen() )
	{
		throw SocketException ( "Could not listen to socket." );
	}

}

}

