/** \file Signal_Manager.cpp  
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "Signal_Manager.h"

namespace rtaixml_domain {


Signal_Manager::Signal_Manager(int Number_Scopes,Target_Scopes_T *T_Scopes)
{
	Scopes=T_Scopes;
	Num_Scopes = Number_Scopes;
	Save_Flag = new int[Num_Scopes];
	Dec_Flag = new int[Num_Scopes];
	ResetAll();
	SetAllDec();
}

Signal_Manager::~Signal_Manager()
{
	delete[] Save_Flag;
	delete[] Dec_Flag;
}

int Signal_Manager::Get_Signal(int index)
{
	return Save_Flag[index];
}

void Signal_Manager::Start_Signal(int index)
{
	if(index>=0 && index<Num_Scopes)
		Save_Flag[index]=1;
}

void Signal_Manager::Stop_Signal(int index)
{
	if(index>=0 && index<Num_Scopes)
		Save_Flag[index]=0;
}

void Signal_Manager::ResetAll()
{
	for (int i = 0; i < Num_Scopes; i++)
		Save_Flag[i] = 0;
}

void Signal_Manager::SetAll()
{
	for (int i = 0; i < Num_Scopes; i++)
		Save_Flag[i] = 1;
}

unsigned int Signal_Manager::SignalNumber()
{
	int active_scopes=0;
	for (int i = 0; i < Num_Scopes; i++)
		if(Save_Flag[i] == 1)
			active_scopes++;
	return active_scopes;
}

int Signal_Manager::Get_Dec(unsigned int index)
{
	if(index>=0 && index<(unsigned int) Num_Scopes)
		return Dec_Flag[index];
	return -1;
}

/**
* Change the decimation of a signal
* @param index the number of the signal
* @param dec the decimation (1 all values - 0 is equivalent to 1)
* @return 
*/
int Signal_Manager::Set_Dec(unsigned int index, unsigned int dec)
{
	if(index>=0 && index<(unsigned int) Num_Scopes)
	{
		if(dec>0)
			Dec_Flag[index]=dec;
		else
			Dec_Flag[index]=1;
		return Dec_Flag[index];
	}
	return -1;
}

/**
* Reset the decimation of all signals to 1.
*/
void Signal_Manager::SetAllDec()
{
	for (int i = 0; i < Num_Scopes; i++)
		Dec_Flag[i] = 1;
}

} //end namespace rtaixml_domain

